% plot_FrankenBedAdj_vs_observations

clear
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));

%% Load FrankenBedAdj
adj = ncread('../out/pub/derived/FrankenBedAdj.nc','Adj_all_gates');
gate_x = ncread('../out/pub/derived/FrankenBedAdj.nc','gate_x');
gate_y = ncread('../out/pub/derived/FrankenBedAdj.nc','gate_y');
num_gates = size(gate_x,1);
migration_distance = 0:200:3000;

%% Load BedMachine v3 and BedMap2
BM = zeros(size(adj)).*NaN;
B2 = zeros(size(adj)).*NaN;
BM_err = zeros(size(adj)).*NaN;
B2_err = zeros(size(adj)).*NaN;
for i = 1%:size(gate_x,1)
    BM(i,:) = wgs2gl04c(gate_x(i,:),gate_y(i,:),bedmachine_interp('bed',gate_x(i,:),gate_y(i,:),'datum','ellipsoid'));
    BM_err(i,:) = bedmachine_interp('errbed',gate_x(i,:),gate_y(i,:));
    B2(i,:) = bedmap2_interp(gate_x(i,:),gate_y(i,:),'bed'); 
    B2_err(i,:) = bedmap2_interp(gate_x(i,:),gate_y(i,:),'beduncertainty'); 
end
FBA = BM - adj; % positive adj means that ice thickness has increased, so we need to lower the bed

%% Load basins
load('../out/int/basins/all_basins.mat');

%% Set example basins to plot
target_basins = {'Thwaites','Pine Island','Flask Glacier','Vincennes Bay','Frost',...
    'Abbot','Baudouin','Cook','Crane','Denman Scott','Ferrigno','Hektoria Green Evans',...
    'Kohler','LarsenD','Lillie','Shackleton','Totten','Tracy Tremenchus'};

%% Prep colours
cmap = cbrewer('qual','Set1',4);
FB_col = 'k';
FBA_col = cmap(2,:);
BM_col = cmap(3,:);
B2_col = cmap(4,:);

%% Plot each target basin
idx = find(basins.type_ID==2);
for i = 1:length(idx)
    %basin_idx = idx(i); % get index of target basin in full basin info
    % At each gate, get the bed
    for j = 1%:num_gates
        tf = inpoly2([gate_x(j,:)',gate_y(j,:)'],[basins.x{idx(i)},basins.y{idx(i)}]); % get location of target basin in gate coordiantes
        % Get basin bed
        basin_FBA = FBA(j,tf);
        basin_adj = adj(j,tf);
        basin_BM = BM(j,tf);
        basin_B2 = B2(j,tf);
        basin_BM_err = BM_err(j,tf);
        basin_B2_err = B2_err(j,tf);
        basin_BM_err(isnan(basin_BM_err)) = abs(basin_BM(isnan(basin_BM_err))).*0.1;
        basin_B2_err(isnan(basin_B2_err)) = abs(basin_B2(isnan(basin_B2_err))).*0.1;
        % Get basin lat/lon
        [basin_lat,basin_lon] = ps2ll(gate_x(j,tf),gate_y(j,tf));
        d = pathdistps(basin_lat,basin_lon,'km');
        % plot
        figure('color','w','units','inches','position',[1 1 4 2],'visible','on')
        sB2 = shadedErrorBar(d,basin_B2,basin_B2_err); hold on
        sBM = shadedErrorBar(d,basin_BM,basin_BM_err); hold on
        plot(d,basin_FBA,'color',FBA_col,'linewidth',1);
        sB2.mainLine.LineWidth = 1; sB2.mainLine.Color = B2_col; sB2.edge(1).Color = 'none'; sB2.edge(2).Color = 'none'; sB2.patch.FaceColor = B2_col; sB2.patch.FaceAlpha = 0.3;
        sBM.mainLine.LineWidth = 1; sBM.mainLine.Color = BM_col; sBM.edge(1).Color = 'none'; sBM.edge(2).Color = 'none'; sBM.patch.FaceColor = BM_col; sBM.patch.FaceAlpha = 0.3;
        box on
        %legend('BedMap2','BedMachine','FrankenBedAdj','location','southeast');
        %title(basins.name{idx(i)});
        ylabel('Elevation (m)');
        xlabel('Distance (km)');
        export_fig(['../out/int/figures/FrankenBedAdj/' basins.name{idx(i)} '_bed_gate_' num2str(migration_distance(j)) 'm.png'],'-dpng','-r450');
        close
    end
end


