% plot_discharge_examples

clear
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));
addpath(genpath('../'));

% basin info
basins = readBasinInfo;
names = basins.name;

% Load data
bQ = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_mean');
bQE = 1.96.*ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_standard_error');
time = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','time')+datenum(1950,1,1,0,0,0);

% add negligible random noise to time to help with plotting
time = time(:)';
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(1,length(time));
time = time + r;
time = sort(time);



%% which shelves to plot

% shelves1 = {'Pine Island','Thwaites'}; % 100-150
% shelves2 = {'Denman Scott','Cook','Frost','Leopold and Astrid Coast'}; % 30-55
% shelves3 = {'Vincennes Bay','Kohler','Jutulstraumen'}; % 20-35
% shelves4 = {'Borchgrevink','Holmes','Ninnis'}; % 19-26
% shelves5 = {'Fleming','Fox','Prospect','Pope','Hektoria Green Evans'}; % 3.5-10
% shelves6 = {'Zonda Eureka','Skelton','Rennick','Leppard'}; % 1-5
% shelves7 = {'Victoria Land2','Lillie','Jorum','Harriott Headland'}; % ~0.2-0.7

% shelves1 = {'Pine Island','Thwaites'}; % 100-150
% shelves2 = {'Dotson_IS','Crosson_IS'}; % 25-40
% shelves3 = {'Riiser and Brunt','Denman Scott'}; % West (a bit too much for this plot)
% shelves4 = {'Frost','Vincennes Bay'}; % Cook (a bit too much discharge for this plot)
% shelves5 = {'Larsen_B_IS','Ferrigno','Dibble'};
% shelves6 = {'Comrie Glacier','Drygalski Glacier'};
% shelves7 = {'Crane Glacier','Erskine Glacier','Flask Glacier','Leay Glacier'};
% shelves8 = {'Nobile Glacier','Vogel Glacier','Boydell Glacier'};
% shelves9 = {'Lawrie Glacier','Birley Glacier','Edgeworth Glacier'}; % Muller Ice Shelf (not quite enough discharge for this plot)
% shelves10 = {'Bleriot Glacier','Niepce Glacier','Luke Glacier','Pyke Glacier'};
% shelves11 = {'Campbell_IS','Bilgeri Glacier','Somigliana Glacier','Eltanin_Bay_IS'};
% shelves12 = {'Balch Glacier','Barnes Glacier','Moser Glacier'};

shelves1 = {'Pine Island','Thwaites','Getz'}; % 100-150
shelves2 = {'Totten','George_VI_IS','West Graham Land','Amery_IS'}; % 75-90
shelves3 = {'Dotson_IS','Crosson_IS','Cook'}; % 31-37
shelves4 = {'Riiser and Brunt','Denman Scott'}; % 41-44
shelves5 = {'Frost','Vincennes Bay','Holmes'}; % 29-33
shelves6 = {'Larsen_B_IS','Ferrigno','Land','Dibble'}; % 14-18
shelves7 = {'Hull','unnamed 1495','Haynes','Rayner Thyer'}; % 10-16
shelves8 = {'Drygalski Glacier','Vigrid','Comrie Glacier','Jackson-Perkins'}; % 2.2-3.1
shelves9 = {'Crane Glacier','Erskine Glacier','Drummond Glacier','Flask Glacier'}; % 1.5-2.2
shelves10 = {'Vogel Glacier','Boydell Glacier_2','Nobile Glacier','Cardell Glacier'}; % 0.05-1.5
shelves11 = {'Birley Glacier','Casey Glacier','Hopkins Glacier','Campbell-Tinker'}; % 0.8-1.1
shelves12 = {'Vallot Glacier','Niepce Glacier','Luke Glacier','Pyke Glacier'};  % 0.7-1.2
shelves13 = {'Richter','Breitfuss Glacier','Northeast Glacier','Bilgeri Glacier'}; % 0.2-0.4
shelves14 = {'Aitkenhead Glacier','Somigliana Glacier','Montgolfier Glacier','unnamed 0215'}; % 0.2-0.35 


%% Plot the full timeseries

% or pastels
cmap = cbrewer('qual','Set2',5);
cmap = repmat(cmap,[12,1]);
col_count = 0;
sc_even = 0;
sc_odd = 0;

% or mass budget scheme
cmap = [0.8500 0.3250 0.0980;
    0.4940 0.1840 0.5560;
    0 0.4470 0.7410;
    0.9290 0.6940 0.1250];

% Make figure
figure('color','w','units','inches','position',[1 1 8 6],'visible','on');
subplot = @(m,n,p) subtightplot (m, n, p,[.01 0.01],[.05 .05],[.1 .1]);

for i = 1:14

    if i == 1
        shelves_to_plot = shelves1;
    elseif i == 2
        shelves_to_plot = shelves2;
    elseif i == 3
        shelves_to_plot = shelves3;
    elseif i == 4
        shelves_to_plot = shelves4;
    elseif i == 5
        shelves_to_plot = shelves5;
    elseif i == 6
        shelves_to_plot = shelves6;
    elseif i == 7
        shelves_to_plot = shelves7;
    elseif i == 8
        shelves_to_plot = shelves8;
    elseif i == 9
        shelves_to_plot = shelves9;
    elseif i == 10
        shelves_to_plot = shelves10;
    elseif i == 11
        shelves_to_plot = shelves11;
    elseif i == 12
        shelves_to_plot = shelves12;
    elseif i == 13
        shelves_to_plot = shelves13;
    elseif i == 14
        shelves_to_plot = shelves14;
    end
    
    p = subplot(7,2,i);
    
    for j = 1:numel(shelves_to_plot)
        % increment colour
        col_count = j;
        % Get index of shelf in full basins
        if strcmp(shelves_to_plot{j},'Riiser and Brunt')
            idx = find(strcmp('Brunt Stancomb',names),1,'first');
            idx2 = find(strcmp('Riiser-Larsen',names),1,'first');
            bQ(idx,:) = bQ(idx,:) + bQ(idx2,:);
            bQE(idx,:) = sqrt(bQE(idx,:).^2 + bQE(idx2,:).^2);
        else
            idx = find(strcmp(shelves_to_plot{j},names),1,'first');
        end
        % plot
        scatter(time,bQ(idx,:),8,cmap(col_count,:),'filled','markerEdgeColor','none'); hold on; datetick('x')
        ptch = shadedErrorBar(time,bQ(idx,:),bQE(idx,:)); % 95% confidence bounds
        ptch.mainLine.Color = 'none';
        ptch.edge(1).Color = 'none';
        ptch.edge(2).Color = 'none';
        ptch.patch.FaceColor = cmap(col_count,:);
        ptch.patch.FaceAlpha = 0.3;
    end
    
    % Visuals
    box on
    set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
        'XTick',datenum(1996:4:2024,1,1),'XTickLabel',{' '});
    pos = p.Position;
    if rem(i,2) == 0
        sc_even = sc_even+1;
        p.Position = [pos(1)+0.1 pos(2)+0.035+((sc_even-1)*0.035) pos(3).*0.8 pos(4)*0.75];
        %set(gca,'YAxisLocation','right');
    else
        sc_odd = sc_odd+1;
        p.Position = [pos(1) pos(2)+0.035+((sc_odd-1)*0.035) pos(3).*0.8 pos(4)*0.75];
    end
    
    if i == 1
        set(gca,'YTick',100:20:140,'YLim',[81 159]); % PIG etc
    elseif i == 2
        set(gca,'YTick',75:5:90,'YLim',[72.99 90.95],'YTickLabel',{'75','80','85','90'}); % Totten etc
    elseif i == 3
        set(gca,'YTick',28:3:40,'YLim',[28.5 39.9]); % Dotson
    elseif i == 4 
        set(gca,'YTick',41:44,'YTickLabel',{'41','42','43','44'},'YLim',[40.8 44.2]); % Brunt
    elseif i == 5
        set(gca,'YTick',24:4:32,'YTickLabel',{'24','28','32'},'YLim',[24.3 33.7]); % Frost
    elseif i == 6
        set(gca,'YTick',13:18,'YTickLabel',{' ','14',' ','16',' ','18'},'YLim',[13.2 18.5]); % Larsen B
    elseif i == 7
        set(gca,'YTick',11:2:15,'YLim',[10.2 16.3],'YTickLabel',{'11','13','15'}); % Haynes
    elseif i == 8
        set(gca,'YTick',2:.5:3.5,'YTickLabel',{'2','2.5','3',' '},'YLim',[1.89 3.3]); % Comrie
    elseif i == 9
        set(gca,'YTick',1.6:.3:2.4,'YTickLabel',{'1.6','1.9','2.2'},'YLim',[1.39 2.34]); % Crane
    elseif i == 10
        set(gca,'YTick',0.05:.05:0.25,'YTickLabel',{' ','0.1',' ','0.2',' '},'YLim',[0.035 0.24]); % Boydell etc
    elseif i == 11
        set(gca,'YTick',0.8:.15:1.25,'YTickLabel',{'0.8','0.95','1.1'},'YLim',[0.68 1.17]); % Lawrie etc
    elseif i == 12
        set(gca,'YTick',0.5:0.25:1,'YTickLabel',{'','0.75','1'},'YLim',[0.56 1.22]); % Niepce etc
    elseif i == 13
        set(gca,'YTick',0.2:0.1:0.4,'YTickLabel',{'0.2','0.3','0.4'},'YLim',[0.178 0.45]); % Breitfuss etc
    elseif i == 14
        set(gca,'YTick',0.2:0.05:0.35,'YTickLabel',{'0.2','0.25','0.3',' '},'YLim',[0.195 0.335]); % Montgolfier etc
    end
    
    if i==13 || i == 14
        set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
        'XTick',datenum(1996:2:2024,1,1),...
        'XTickLabel',{'','1998','','2002','','2006','','2010','','2014','','2018','','2022',''}); 
        xtickangle(45)
    end

end


% save
export_fig('../out/pub/figures/discharge_examples/discharge_examples_1996-2023.png','-dpng','-r1200');








% %% Plot the timeseries after 2016
% 
% col_count = 0;
% sc_even = 0;
% sc_odd = 0;
% 
% % Make figure
% figure('color','w','units','inches','position',[1 1 2 6],'visible','on');
% subplot = @(m,n,p) subtightplot (m, n, p,[.01 0.01],[.05 .05],[.15 .05]);
% 
% for i = 1:6
% 
%     if i == 1
%         shelves_to_plot = shelves1;
%     elseif i == 2
%         shelves_to_plot = shelves2;
%     elseif i == 3
%         shelves_to_plot = shelves3;
%     elseif i == 4
%         shelves_to_plot = shelves4;
%     elseif i == 5
%         shelves_to_plot = shelves5;
%     elseif i == 6
%         shelves_to_plot = shelves6;
%     elseif i == 7
%         shelves_to_plot = shelves7;
%     elseif i == 8
%         shelves_to_plot = shelves8;
%     end
%     
%     p = subplot(6,1,i);
%     
%     for j = 1:numel(shelves_to_plot)
%         
%         % increment colour
%         col_count = col_count+1;
%         col_count = j;
%         
%         % Get index of shelf in full basins
%         idx = find(strcmp(shelves_to_plot{j},basins.name),1,'first');
%     
%         % plot
%         scatter(time,FB_mean(idx,:),8,cmap(col_count,:),'filled','markerEdgeColor','none'); hold on; datetick('x')
%         ptch = shadedErrorBar(time,FB_mean(idx,:),1.96.*FB_se(idx,:)); % 95% confidence bounds
%         ptch.mainLine.Color = 'none';
%         ptch.edge(1).Color = 'none';
%         ptch.edge(2).Color = 'none';
%         ptch.patch.FaceColor = cmap(col_count,:);
%         ptch.patch.FaceAlpha = 0.3;
%         % plot the unsmoothed data
%         %plot(time,ns.bQ(idx,:),'color',cmap(col_count,:),'linewidth',1); 
%         disp(['Just plotted ' shelves_to_plot{j} ]);
%         disp(' ');
%                 
%     end
%     
%     box on
%     set(gca,'XLim',[datenum(2016,1,1) datenum(2023,7,1)],...
%         'XTick',datenum(2017:2:2023,1,1),'XTickLabel',{' '});
%     pos = p.Position;
% %     if rem(i,2) == 0
% %         sc_even = sc_even+1;
% %         p.Position = [pos(1) pos(2)+0.04+((sc_even-1)*0.04) pos(3) pos(4)*0.75];
% %         set(gca,'YAxisLocation','right');
% %     else
%         sc_odd = sc_odd+1;
%         p.Position = [pos(1) pos(2)+0.04+((sc_odd-1)*0.04) pos(3) pos(4)*0.75];
%     %end
%     
%     if i == 1
%         set(gca,'YLim',[128 155]);
%     elseif i == 2
%         set(gca,'YTick',32:2:40,'YLim',[33 41]);
%     elseif i ==3
%         set(gca,'YTick',14:2:22);
%     elseif i == 4
%         set(gca,'YTick',1.25:0.5:2.75);
%     elseif i == 5
%         set(gca,'YTick',0.6:0.2:1);
%     elseif i == 6
%         set(gca,'YTick',0.2:0.1:0.6,'YTickLabel',{' ','0.3','0.4','0.5',' '});
%     end
%     
%     if i==6 || i == 6
%         set(gca,'XTickLabel',{'2017','2019','2021','2023'}); 
%         xtickangle(45)
%     end
% 
% end
% 
% 
% 
% % save
% export_fig('../out/pub/figures/discharge_examples/discharge_examples_2016-2023.png','-dpng','-r600');

