% plot_gate_impact

clear
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));
addpath(genpath('../'));

% basin info
basins = readBasinInfo;
num_gates = 16;

% Load data
time = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','time') + datenum(1950,1,1,0,0,0);
Q = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_all_gates');
Qavg = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_mean');

% Calculate relative difference between gates
gate_diff = (Q-Qavg)./Qavg.*100;

% Prepare plotting
[tg,gt] = meshgrid(time,0:200:3000);

%% Plot

% Make figure
figure('color','w','units','inches','position',[1 1 8 6],'visible','on');
subplot = @(m,n,p) subtightplot (m, n, p,[.01 0.01],[.05 .05],[.1 .1]);

% Plot all Antarctica
p1 = subplot(5,1,1);
%imagesc(time,0:200:3000,squeeze(gate_diff(ind,:,:))'); datetick('x');
pcolor(tg,gt,squeeze(gate_diff(1,:,:))'); datetick('x'); shading interp;
colormap(p1,cmocean('balance'));
caxis([-5 5]);
box on
set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
    'XTick',datenum(1996:2:2024,1,1),'XTickLabel',{' '});
pos = p1.Position;
p1.Position = [pos(1) pos(2) pos(3) pos(4)*0.75];
text(datenum(1998,7,1),2500,'Antarctica','color','k','fontname','Helvetica','fontweight','bold');

% West Antarctica
p2 = subplot(5,1,2);
%imagesc(time,0:200:3000,squeeze(gate_diff(2,:,:))'); datetick('x');
pcolor(tg,gt,squeeze(gate_diff(2,:,:))'); datetick('x'); shading interp;
colormap(p2,cmocean('balance'));
caxis([-5 5]);
box on
set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
    'XTick',datenum(1996:2:2024,1,1),'XTickLabel',{' '},...
    'YAxisLocation','right');
pos = p2.Position;
p2.Position = [pos(1) pos(2)+0.035 pos(3) pos(4)*0.75];
text(datenum(1998,7,1),2500,'West Antarctica','color','k','fontname','Helvetica','fontweight','bold');

% East Antarctica
p3 = subplot(5,1,3);
%imagesc(time,0:200:3000,squeeze(gate_diff(3,:,:))'); datetick('x');
pcolor(tg,gt,squeeze(gate_diff(3,:,:))'); datetick('x'); shading interp;
colormap(p3,cmocean('balance'));
caxis([-5 5]);
box on
set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
    'XTick',datenum(1996:2:2024,1,1),'XTickLabel',{' '});
pos = p3.Position;
p3.Position = [pos(1) pos(2)+0.07 pos(3) pos(4)*0.75];
text(datenum(1998,7,1),2500,'East Antarctica','color','k','fontname','Helvetica','fontweight','bold');

% Peninsula
p4 = subplot(5,1,4);
%imagesc(time,0:200:3000,squeeze(gate_diff(ind,:,:))'); datetick('x');
pcolor(tg,gt,squeeze(gate_diff(4,:,:))'); datetick('x'); shading interp;
colormap(p4,cmocean('balance'));
caxis([-5 5]);
box on
set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
    'XTick',datenum(1996:2:2024,1,1),...
    'XTickLabel',{'1996','1998','2000','2002','2004','2006','2008','2010',...
    '2012','2014','2016','2018','2020','2022','2024'},...
    'YAxisLocation','right');
pos = p4.Position;
p4.Position = [pos(1) pos(2)+0.105 pos(3) pos(4)*0.75];
text(datenum(1998,7,1),2500,'Peninsula','color','k','fontname','Helvetica','fontweight','bold');

export_fig('../out/pub/figures/gate_impact/gate_discharge_diff.png','-dpng','-r600');
close

%% Plot one with a colorbar
% Make figure
figure('color','w','units','inches','position',[1 1 8 6],'visible','on');
subplot = @(m,n,p) subtightplot (m, n, p,[.01 0.01],[.05 .05],[.1 .1]);

% Plot all Antarctica
p1 = subplot(3,1,1);
ind = 1;
% for i = 1:num_gates
%     scatter(time,squeeze(gate_diff(ind,:,i)),8,cmap(i,:),'filled','markerEdgeColor','none'); 
%     if i == 1; hold on; datetick('x'); end
% end
%imagesc(time,0:200:3000,squeeze(gate_diff(ind,:,:))'); datetick('x');
pcolor(tg,gt,squeeze(gate_diff(ind,:,:))'); datetick('x'); shading interp;
colormap(p1,cmocean('balance'));
caxis([-5 5]);
box on
set(gca,'XLim',[datenum(1996,1,1) datenum(2024,1,1)],...
    'XTick',datenum(1996:2:2024,1,1),'XTickLabel',{' '});
pos = p1.Position;
p1.Position = [pos(1) pos(2)+0.05 pos(3) pos(4)*0.75];
text(datenum(1998,7,1),2500,'Antarctica','color','k','fontname','Helvetica','fontweight','bold');
cb = colorbar;
caxis([-5 5]);
colormap(cmocean('balance'));
export_fig('../out/pub/figures/gate_impact/gate_colorbar.png','-dpng','-r600');
close

