% calc_gate_to_GL_area

clear 
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));

%% Grounding line

% Start with the MEaSUREs grounding line
disp('loading initial grounding line...');
[gl_x,gl_y] = antbounds_data('gl','xy'); 

% Get indices of main ice sheet (not islands), so that we can remove them or adjust the
% position of those pixels independently from the islands
basins = shaperead('Basins_IMBIE_Antarctica_v02.shp');
islands_x = basins(1).X;
islands_y = basins(1).Y;
[~,main] = inpoly2([gl_x(:),gl_y(:)],[islands_x(:),islands_y(:)]);
main = logical(1-main); % invert
clear islands_x islands_y basins

% Remove islands 
gl_x = gl_x(main);
gl_y = gl_y(main);
clear main

% Before adjusting the position of the gate, increase the density of grounding
% line points, so that the migration is a bit smoother
disp('increasing density of initial grounding line points...');
s = regionprops(isfinite(gl_x),gl_x,'PixelIdxList');
region_x = cell(numel(s),1);
region_y = cell(numel(s),1);
for i = 1:numel(s)
    tmp_x = gl_x(s(i).PixelIdxList);
    tmp_y = gl_y(s(i).PixelIdxList);
    total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
    xy = interparc(0:(250/total_length):1,tmp_x(:),tmp_y(:),'linear');
    region_x{i} = xy(:,1);
    region_y{i} = xy(:,2);
end
gl_x = cell2nancat(region_x);
gl_y = cell2nancat(region_y);
clear region_x region_y total_length xy tmp_x tmp_y i s

%% Load flux gates
load('../out/pub/gates/flux_gates_migrated.mat');
migration_distance = 0:200:3000; % metres

%% Load basins
load('../out/int/basins/all_basins.mat');

%% Load polygons
% Code to make them (and see gate2poly)
% poly_GL = polyshape(gl_x,gl_y);
% for i = 1:numel(gate_x_migrate)
%     poly_gate(i) = polyshape(gate_x_migrate{i},gate_y_migrate{i});
% end
load('../out/int/gate_data/poly_GL.mat');
for i = 1:numel(gate_x_migrate)
    tmp = load(['../out/int/gate_data/poly_gate_' num2str(migration_distance(i)) 'm.mat']);
    poly_gate(i) = tmp.poly_gate;
end

%% Get intervening region
disp('Defining intervening region...')
% for i = 1:numel(gate_x_migrate)
%     disp(['Processing gate ' num2str(i)])
%     poly_gap(i) = xor(poly_GL,poly_gate(i));
% end
load('../out/int/gate_data/poly_gap.mat');

%% Subset by basin
disp('Calculating area for each basin');
gap_area = zeros([numel(basins.name),numel(gate_x_migrate)]).*NaN;
for i = 1:numel(basins.name)
    disp(['Processing basin ' num2str(i) '/' num2str(numel(basins.name))])
    basin = polyshape(basins.x{i},basins.y{i});
    for j = 1:numel(gate_x_migrate)
        p = intersect(poly_gap(j),basin);
        gap_area(i,j) = area(p); % meters squared
    end
end

%% Save
save('gate_to_GL_area.mat','gap_area');

