% calc_smb_mass_correction

clear 
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));

%% Get long-term average SMB
% Get info 
fn = 'RACMO2.3p2_ERA5_ANT27_smb_monthly_1979_01_2021_12.nc';
smb_time = datenum(1950,1,1) + ncread(fn,'time');
lat = ncread(fn,'lat');
lon = ncread(fn,'lon');
[x,y] = ll2ps(lat,lon);
clear lat lon 

% Load
smb = ncread(fn,'smb');
smb(smb==-9999) = NaN;
smb(smb<0.01 & smb>-0.01)=NaN;
smb = permute(smb,[1,2,4,3]);

% Get long-term mean in every pixel
idx = find(smb_time>=datenum(1979,1,1) & smb_time<datenum(2009,1,1));
smb_avg = nanmean(smb(:,:,idx),3);
nanval = smb_avg(1);
smb_avg(smb_avg==nanval) = NaN;

% Fill gaps in long term average
smb_avg([1,end],:) = 0;
smb_avg(:,[1,end]) = 0;
smb_avg = regionfill(smb_avg,isnan(smb_avg));

%% Prep
% Load polygons of gate-to-grounding line regions
load('../out/int/gate_data/poly_gap.mat'); % see function at bottom of script

% Load basins
load('../out/int/basins/all_basins.mat');

%% Integrate

% Create and interpolant for the SMB data
F = scatteredInterpolant(x(:),y(:),smb_avg(:));

% Get the poly_gap within each basin
num_gates = 16;
smb_mass = zeros([numel(basins.name),num_gates]);
for i = 1:numel(basins.name)
    disp(['Processing basin ' num2str(i) '/' num2str(numel(basins.name))])
    basin = polyshape(basins.x{i},basins.y{i});
    for j = 1:num_gates
        p = intersect(poly_gap(j),basin); % get the intersection of the poly_gap and the basin
        gap_area = area(p); % meters squared
        if gap_area>0
            smb_mass(i,j) = (((nanmean(F(p.Vertices(:,1),p.Vertices(:,2))) / (365.25/12))*365.25) * gap_area) *1e-12; % Gt/yr; positive is SMB-induced thickening
        end
    end
end

%% Save
save('../out/int/gate_data/smb_mass_correction.mat','smb_mass');



% %% make_polygap
% 
% %%% Grounding line
% 
% % Start with the MEaSUREs grounding line
% disp('loading initial grounding line...');
% [gl_x,gl_y] = antbounds_data('gl','xy'); 
% 
% % Get indices of main ice sheet (not islands), so that we can remove them or adjust the
% % position of those pixels independently from the islands
% basins = shaperead('Basins_IMBIE_Antarctica_v02.shp');
% islands_x = basins(1).X;
% islands_y = basins(1).Y;
% [~,main] = inpoly2([gl_x(:),gl_y(:)],[islands_x(:),islands_y(:)]);
% main = logical(1-main); % invert
% clear islands_x islands_y basins
% 
% % Remove islands 
% gl_x = gl_x(main);
% gl_y = gl_y(main);
% clear main
% 
% % Before adjusting the position of the gate, increase the density of grounding
% % line points, so that the migration is a bit smoother
% disp('increasing density of initial grounding line points...');
% s = regionprops(isfinite(gl_x),gl_x,'PixelIdxList');
% region_x = cell(numel(s),1);
% region_y = cell(numel(s),1);
% for i = 1:numel(s)
%     tmp_x = gl_x(s(i).PixelIdxList);
%     tmp_y = gl_y(s(i).PixelIdxList);
%     total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
%     xy = interparc(0:(250/total_length):1,tmp_x(:),tmp_y(:),'linear');
%     region_x{i} = xy(:,1);
%     region_y{i} = xy(:,2);
% end
% gl_x = cell2nancat(region_x);
% gl_y = cell2nancat(region_y);
% clear region_x region_y total_length xy tmp_x tmp_y i s
% 
% %%% Load flux gates
% load('../out/pub/gates/flux_gates_migrated.mat');
% migration_distance = 0:200:3000; % metres
% 
% %%% Load basins
% load('../out/int/basins/all_basins.mat');
% 
% %%% Load polygons
% poly_GL = polyshape(gl_x,gl_y);
% for i = 1:numel(gate_x_migrate)
%     poly_gate(i) = polyshape(gate_x_migrate{i},gate_y_migrate{i});
% end
% 
% %%% Get intervening region
% disp('Defining intervening region...')
% for i = 1:numel(gate_x_migrate)
%     disp(['Processing gate ' num2str(i)])
%     poly_gap(i) = xor(poly_GL,poly_gate(i));
% end
