function compile_ice_velocity(migration_distance)

% add paths to libraries
if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
    addpath(genpath('/resstore/b0133/Data/Velocity'));
end

%% Load flux gate

g = load('../out/pub/gates/flux_gates_migrated.mat');
migration_distances = 0:200:3000; % metres
migration_pixels = find(migration_distances == migration_distance); % metres
gate_x = g.gate_x_migrate{migration_pixels};
gate_y = g.gate_y_migrate{migration_pixels};
gate_nan = isnan(gate_x); % indices of gate nans
clear g migration_pixels migration_distances


%% Load velocity 

% MEaSUREs annual data, 1x1 km
%   - get measures coordinates
filename = ['Antarctica_ice_velocity_2000_2001_1km_v01.1.nc'];
xM = ncread(filename,'x');
yM = ncread(filename,'y');
%   - pre-allocate output
spM = zeros([length(gate_x),length([2000,2005:2019])]).*NaN;
%vyM = zeros([length(gate_x),length([2000,2005:2019])]).*NaN;
spM_err = zeros([length(gate_x),length([2000,2005:2019])]).*NaN;
%vyM_err = zeros([length(gate_x),length([2000,2005:2019])]).*NaN;
timeM = datenum([2000,2005:2019],12,30); % annual products are averaged between July 1st and June 30th each year. Dec 30th is mid-date in non-leap year
sourceM = ones([length([2000,2005:2019]),1]);
counter = 1;
% loop through annual products and extract
for i = [2000,2005:2019] % these are available up to 2020 now, but we have higher resolution options after 2015 
    if i == 2000 || i >= 2017
        filename = ['Antarctica_ice_velocity_' num2str(i) '_' num2str(i+1) '_1km_v01.1.nc'];
    else
        filename = ['Antarctica_ice_velocity_' num2str(i) '_' num2str(i+1) '_1km_v01.nc'];
    end
    % speed
    vx = double(flipud(rot90(ncread(filename,'VX'))));
    vy = double(flipud(rot90(ncread(filename,'VY'))));
    data = hypot(vx,vy);
    data = interp2(xM,yM,data,gate_x,gate_y,'linear');
    spM(:,counter) = data;
    % error
    vx = double(flipud(rot90(ncread(filename,'ERRX'))));
    vy = double(flipud(rot90(ncread(filename,'ERRY')))); 
    data = hypot(vx,vy);
    data = interp2(xM,yM,data,gate_x,gate_y,'linear');
    spM_err(:,counter) = data;
    % increment counter
    counter=counter+1;
end

% ENVEO monthly data
%   - get filelist
if ispc
    nc_list = dir('D:/work/Fellowship_Leeds/Data/Velocity/Antarctica/ENVEO/monthly/*.nc');
else
    nc_list = dir('/resstore/b0133/Data/Velocity/ENVEO/monthly/*.nc');
end
%   - Get mid-date of all netcdf files
timeS1 = zeros([numel(nc_list),1]).*NaN;
for i = 1:numel(nc_list)
    nc_file = nc_list(i).name;
    Y = str2double(nc_file(1:4));
    M = str2double(nc_file(5:6));
    D = str2double(nc_file(7:8));
    timeS1(i) = datenum(Y,M,D) + (datenum(Y,M+1,D)-datenum(Y,M,D))/2;
end
%   - Load coordinates and get index of gate locations
xS1 = ncread([nc_list(i).folder '/' nc_list(i).name],'x');
yS1 = ncread([nc_list(i).folder '/' nc_list(i).name],'y');
%   - pre-allocate output
spS1 = zeros([length(gate_x),numel(nc_list)]).*NaN;
spS1_err = zeros([length(gate_x),numel(nc_list)]).*NaN;
sourceS1 = ones([numel(nc_list),1]).*2;
%   - Loop through nc files and load data at gate locations
for i = 1:numel(nc_list)
    % speed
    data = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'land_ice_surface_velocity_magnitude'))));
    data(abs(data)>20000) = NaN;
    data = data.*365.25; % convert to m/yr
    data = interp2(xS1,yS1,data,gate_x,gate_y,'linear');
    spS1(:,i) = data;
    % % Vy
    % data = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'land_ice_surface_northing_velocity'))));
    % data(abs(data)>20000) = NaN;
    % data = data.*365.25; % convert to m/yr
    % data = interp2(xS1,yS1,data,gate_x,gate_y,'linear');
    % vyS1(:,i) = data;
    % vx error
    vx = double(rot90(flipud(ncread([nc_list(i).folder '/' nc_list(i).name],'land_ice_surface_easting_stddev'))));
    vx(abs(vx)>20000) = NaN;
    vx = vx.*365.25; % convert to m/yr
    vx = interp2(xS1,yS1,vx,gate_x,gate_y,'linear');
    % % Vy error
    vy = double(rot90(flipud(ncread([nc_list(i).folder '/' nc_list(i).name],'land_ice_surface_northing_stddev'))));
    vy(abs(vy)>20000) = NaN;
    vy = vy.*365.25; % convert to m/yr
    vy = interp2(xS1,yS1,vy,gate_x,gate_y,'linear');
    % sp error
    data = hypot(vx,vy);
    spS1_err(:,i) = data;
end

% Merged 1996 data
%   - load
load('ASE_1996/ASE_1996_merged.mat')
vx_1996 = flipud(vx_1996);
vy_1996 = flipud(vy_1996);
%   - extract at gate
sp = hypot(vx_1996,vy_1996);
spERS = interp2(x_1996,y_1996,sp,gate_x,gate_y,'linear');
timeERS = datenum(1996,1,1) + 365/2;
spERS_err = abs(spERS).*0.1; % assume 10% velocity error
sourceERS = 3;

% ITSLIVE annual velocity
%   - get filelist
if ispc
    nc_list1 = dir('D:\work\Fellowship_Leeds\Data\Velocity\Antarctica\ITSLIVE/*_19*.nc');
    nc_list2 = dir('D:\work\Fellowship_Leeds\Data\Velocity\Antarctica\ITSLIVE/*_20*.nc');
    nc_list = [nc_list1;nc_list2];
else
    nc_list1 = dir('/resstore/b0133/Data/Velocity/ITSLIVE/*_199*.nc');
    nc_list2 = dir('/resstore/b0133/Data/Velocity/ITSLIVE/*_20*.nc');
    nc_list = [nc_list1;nc_list2];
    clear nc_list1 nc_list2
end
%   - Get mid-date of all netcdf files
timeI = zeros([numel(nc_list),1]).*NaN;
for i = 1:numel(nc_list)
    nc_file = nc_list(i).name;
    Y = str2double(nc_file(11:14));
    timeI(i) = datenum(Y,1,1) + (datenum(Y,12,31)-datenum(Y,1,1))/2;
end
%   - Load coordinates and get index of gate locations
xI = ncread([nc_list(i).folder '/' nc_list(i).name],'x');
yI = ncread([nc_list(i).folder '/' nc_list(i).name],'y');
%   - pre-allocate output
spI = zeros([length(gate_x),length(timeI)]).*NaN;
spI_err = zeros([length(gate_x),length(timeI)]).*NaN;
%   - Loop through nc files and load data at gate locations
sourceI = ones([length(timeI),1]).*4;
for i = 1:numel(nc_list)
    % speed
    vx = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'vx'))));
    vx(abs(vx(:))>20000) = NaN;
    vy = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'vy'))));
    vx(abs(vy(:))>20000) = NaN;
    data = hypot(vx,vy);
    spI(:,i) = interp2(xI,yI,data,gate_x,gate_y,'linear');
    % error
    vx = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'vx_err'))));
    vx(abs(vx)>20000) = NaN;
    vy = double(flipud(rot90(ncread([nc_list(i).folder '/' nc_list(i).name],'vy_err'))));
    vy(abs(vy)>20000) = NaN;
    data = hypot(vx,vy);
    spI_err(:,i) = interp2(xI,yI,data,gate_x,gate_y,'linear');
end

% MEaSUREs 1995-2001 mosaic
%   - info
filename = 'antarctica_ice_velocity_1995-2001_450m_v01.0.nc';
timeM95 = datenum(1995,7,1) + (datenum(2001,6,30)-datenum(1995,7,1))/2;
xM95 = ncread(filename,'x');
yM95 = ncread(filename,'y');
%   - vx
data = ncread(filename,'VX');
data = flipud(double(rot90(data)));
vx = interp2(xM95,yM95,data,gate_x,gate_y,'linear');
%   - vy
data = ncread(filename,'VY');
data = flipud(double(rot90(data)));
vy = interp2(xM95,yM95,data,gate_x,gate_y,'linear');
% speed
spM95 = hypot(vx,vy);
% vx err
data = ncread(filename,'ERRX');
data = flipud(double(rot90(data)));
vx = interp2(xM95,yM95,data,gate_x,gate_y,'linear');
% vy err
data = ncread(filename,'ERRY');
data = flipud(double(rot90(data)));
vy = interp2(xM95,yM95,data,gate_x,gate_y,'linear');
% error
spM95_err = hypot(vx,vy);
sourceM95 = 5;

% MEaSUREs 2007-2009 mosaic
%   - info
filename = 'antarctica_ice_velocity_2007-2009_450m_v01.0.nc';
timeM08 = datenum(2007,7,1) + (datenum(2009,6,30)-datenum(2007,7,1))/2;
xM08 = ncread(filename,'x');
yM08 = ncread(filename,'y');
%   - vx
data = ncread(filename,'VX');
data = flipud(double(rot90(data)));
vx = interp2(xM08,yM08,data,gate_x,gate_y,'linear');
%   - vy
data = ncread(filename,'VY');
data = flipud(double(rot90(data)));
vy = interp2(xM08,yM08,data,gate_x,gate_y,'linear');
% speed
spM08 = hypot(vx,vy);
% vx err
data = ncread(filename,'ERRX');
data = flipud(double(rot90(data)));
vx = interp2(xM08,yM08,data,gate_x,gate_y,'linear');
% vy err
data = ncread(filename,'ERRY');
data = flipud(double(rot90(data)));
vy = interp2(xM08,yM08,data,gate_x,gate_y,'linear');
% error
spM08_err = hypot(vx,vy);
sourceM08 = 5;


% MEaSUREs 2014-2017 mosaic
%   - info
filename = 'antarctica_ice_velocity_2014-2017_450m_v01.0.nc';
timeM15 = datenum(2014,7,1) + (datenum(2017,6,30)-datenum(2014,7,1))/2;
xM15 = ncread(filename,'x');
yM15 = ncread(filename,'y');
%   - vx
data = ncread(filename,'VX');
data = flipud(double(rot90(data)));
vx = interp2(xM15,yM15,data,gate_x,gate_y,'linear');
%   - vy
data = ncread(filename,'VY');
data = flipud(double(rot90(data)));
vy = interp2(xM15,yM15,data,gate_x,gate_y,'linear');
% speed
spM15 = hypot(vx,vy);
% vx err
data = ncread(filename,'ERRX');
data = flipud(double(rot90(data)));
vx = interp2(xM15,yM15,data,gate_x,gate_y,'linear');
% vy err
data = ncread(filename,'ERRY');
data = flipud(double(rot90(data)));
vy = interp2(xM15,yM15,data,gate_x,gate_y,'linear');
% error
spM15_err = hypot(vx,vy);
sourceM15 = 5;


% SID ice velocity
disp('Processing SID');
%load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm.mat'],'gate_fd');
SID_list = dir('/resstore/b0133/SID/Antarctica/IV/S1/output/mosaic/monthly_corrected/**/sp_corrected.tif');
% Get times
SID_dates = zeros([numel(SID_list),2]).*NaN;
for i = 1:numel(SID_list)
    d = extractAfter(SID_list(i).folder,'monthly_corrected/');
    SID_dates(i,:) = [datenum(d(1:8),'yyyymmdd') , datenum(d(10:17),'yyyymmdd')];
end
timeSID = nanmean(SID_dates,2);
% Get image coordinates
[~,xSID,ySID] = geoimread([SID_list(1).folder '/vx_corrected.tif']);
% Pre-allocate output
spSID = zeros([length(gate_x),numel(SID_list)]).*NaN;
spSID_err = zeros([length(gate_x),numel(SID_list)]).*NaN;
sourceSID = ones([numel(SID_list),1]).*6;
% Loop through new files and extract at gate
for i = 1:numel(SID_list)
    disp(['Processing month ' num2str(i) ' of ' num2str(numel(SID_list)) ]);
    % vx
    data = double(geoimread([SID_list(i).folder '/sp_corrected.tif']));
    spSID(:,i) = interp2(xSID,ySID,data,gate_x,gate_y,'linear');
    % error
    data = double(geoimread(['/resstore/b0133/SID/Antarctica/IV/S1/output/mosaic/monthly/' datestr(SID_dates(i,1),'yyyymmdd') '_' datestr(SID_dates(i,2),'yyyymmdd') '/err_clean.tif']));
    spSID_err(:,i) = interp2(xSID,ySID,data,gate_x,gate_y,'linear');
    %[tmp_vx,tmp_vy] = compass2cart(gate_fd(isfinite(sp_err)),sp_err(isfinite(sp_err)));
    %[tmp_vx,tmp_vy] = uv2vxvy(gate_x(isfinite(sp_err)),gate_y(isfinite(sp_err)),tmp_vx,tmp_vy);
    %vxSID_err(isfinite(sp_err),i) = abs(tmp_vx);
    %vySID_err(isfinite(sp_err),i) = abs(tmp_vy);
end
clear tmp_vx tmp_vy sp_err data xSID ySID


% ITSLIVE ASE simulated velocity
%   - get filelist
Isim_list = dir('/resstore/b0133/Data/Velocity/ITSLIVE_ASE_sim/*_vx.tif');
%   - Get mid-date of all netcdf files
timeIsim = zeros([numel(Isim_list),1]).*NaN;
for i = 1:numel(Isim_list)
    sim_file = Isim_list(i).name;
    Y = str2double(sim_file(9:12));
    timeIsim(i) = datenum(Y,1,1) + (datenum(Y,12,31)-datenum(Y,1,1))/2;
end
%   - Load coordinates and get index of gate locations
[~,x_sim,y_sim] = geoimread([Isim_list(1).folder '/' Isim_list(1).name]);
[X_sim,Y_sim] = meshgrid(x_sim,y_sim);
%   - pre-allocate output
spIsim = zeros([length(gate_x),length(timeIsim)]).*NaN;
%   - Loop through nc files and load data at gate locations
sourceIsim = ones([length(timeIsim),1]).*8;
for i = 1:numel(Isim_list)
    % Vx
    vx = double(geoimread([Isim_list(i).folder '/' Isim_list(i).name]));
    % Vy
    n = extractBefore(Isim_list(i).name,'vx');
    n = [n 'vy.tif'];
    vy = double(geoimread([Isim_list(i).folder '/' n]));
    data = hypot(vx,vy);
    spIsim(:,i) = interp2(X_sim,Y_sim,data,gate_x,gate_y,'linear');
end
spIsim_err = abs(spIsim).*0.1;


% Joughin velocity data
if ispc
    vx_filelist = dir('../../PIG_mass_budget/Data/joughin2021/RWArchive/velocityTimeSeries/*TSX*.vx.tif'); % quarterly velocity components
    vy_filelist = dir('../../PIG_mass_budget/Data/joughin2021/RWArchive/velocityTimeSeries/*TSX*.vy.tif'); % quarterly velocity components
else
    vx_filelist = dir('/resstore/b0133/Data/Velocity/joughin2021_PIG/*TSX*.vx.tif'); % quarterly velocity components
    vy_filelist = dir('/resstore/b0133/Data/Velocity/joughin2021_PIG/*TSX*.vy.tif'); % quarterly velocity components
end
sp_joughin = zeros([length(gate_x),numel(vx_filelist)]).*NaN;
V_joughin_time = zeros([numel(vx_filelist),1]).*NaN;
joughin_source = ones([numel(vx_filelist),1]).*9;
for i = 1:numel(vx_filelist)
    % Load data
    filename = vx_filelist(i).name;
    % Get date
    if length(filename) == 28 % TSX naming convention
        V_joughin_time(i) = datenum(str2double(filename(12:15)),str2double(filename(17:18)),str2double(filename(20:21)));
    else % S1 naming convention
        V_joughin_time(i) = datenum(str2double(filename(20:23)),str2double(filename(25:26)),str2double(filename(28:29)));
    end
    % vx
    [vx,joughin_x,joughin_y,~] = geoimread([vx_filelist(i).folder '/' vx_filelist(i).name]);
    vx(abs(vx)>20000) = NaN;
    % vy
    [vy,joughin_x,joughin_y,~] = geoimread([vy_filelist(i).folder '/' vy_filelist(i).name]);
    vy(abs(data)>20000) = NaN;
    data = hypot(vx,vy);
    data = interp2(joughin_x,joughin_y,data,gate_x,gate_y,'linear');
    sp_joughin(:,i) = data;
end
% Set error as 10% of velocity
sp_err_joughin = abs(sp_joughin).*0.1;


%% Concatenate velocity measurements

% concatenate
sp_raw = [spM, spS1, spI, spERS, spM95, spM08, spM15, spSID, spIsim, sp_joughin];
spE_raw = [spM_err, spS1_err, spI_err, spERS_err, spM95_err, spM08_err, spM15_err, spSID_err, spIsim_err, sp_err_joughin];
time_vel = [timeM(:)', timeS1(:)', timeI(:)', timeERS, timeM95, timeM08, timeM15, timeSID(:)', timeIsim(:)', V_joughin_time(:)'];
source_vel = [sourceM(:)', sourceS1(:)', sourceI(:)', sourceERS, sourceM95, sourceM08, sourceM15, sourceSID(:)', sourceIsim(:)', joughin_source(:)'];

% Sort
[~,iv] = sort(time_vel,'ascend');
time_vel = time_vel(iv);
sp_raw = sp_raw(:,iv);
spE_raw = spE_raw(:,iv);
source_vel = source_vel(iv);



%% Save raw velocity timeseries

save(['../out/int/gate_data/gate_vel_raw_linear_migrate_' num2str(migration_distance) 'm_compiled.mat'],'sp_raw','spE_raw','source_vel','time_vel');

