% calc_mass_balance
% didn't really get very far with this...

clear
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));
addpath(genpath('../'));

%% Load IMBIE data
IMBIE.time = datenum(xlsread('imbie_antarctica_2021_Gt.csv','A2:A349'),1,1);
regions = {'antarctica','west_antarctica','east_antarctica','antarctic_peninsula'};
for i = 1:4
    IMBIE.MB(i,:) = xlsread(['imbie_' regions{i} '_2021_Gt.csv'],'B2:B349');
    IMBIE.MB_err(i,:) = xlsread(['imbie_' regions{i} '_2021_Gt.csv'],'C2:C349');
    IMBIE.cMB(i,:) = xlsread(['imbie_' regions{i} '_2021_Gt.csv'],'D2:D349');
    IMBIE.cMB_err(i,:) = xlsread(['imbie_' regions{i} '_2021_Gt.csv'],'E2:E349');
end

%% load basins
basins = readBasinInfo;
all_basins = load('../out/int/basins/all_basins.mat');
basin_idx = zeros([numel(basins.name),1]).*NaN;
for i = 1:numel(basins.name)
    basin_idx(i) = find(strcmp(basins.name{i},all_basins.basins.name)==1);
end

%% balance discharge
Qbal = load('../out/int/discharge/balance/balance_discharge.mat');
Qbal.HIRHAM_tf_all = Qbal.HIRHAM_tf_all{basin_idx};
Qbal.MAR_tf_all = Qbal.MAR_tf_all{basin_idx};
Qbal.RACMO_tf_all = Qbal.RACMO_tf_all{basin_idx};
Qbal.Qbal_HIRHAM = Qbal.Qbal_HIRHAM(basin_idx);
Qbal.Qbal_MAR = Qbal.Qbal_MAR(basin_idx);
Qbal.Qbal_RACMO = Qbal.Qbal_RACMO(basin_idx);
Qbal.Qbal_ens = Qbal.Qbal_ens(basin_idx);
Qbal.Qbal_err_HIRHAM = Qbal.Qbal_err_HIRHAM(basin_idx);
Qbal.Qbal_err_MAR = Qbal.Qbal_err_MAR(basin_idx);
Qbal.Qbal_err_RACMO = Qbal.Qbal_err_RACMO(basin_idx);
Qbal.Qbal_err_ens = Qbal.Qbal_err_ens(basin_idx);

%% SMB timeseries (Gt/yr)

% load the SMB anomalies
smb = load('../out/int/derived/basin_smb_anom.mat');
time = smb.smb_time;
smb.HIRHAM_smb_anom = smb.HIRHAM_smb_anom(basin_idx,:);
smb.RACMO_smb_anom = smb.RACMO_smb_anom(basin_idx,:);
smb.MAR_smb_anom = smb.MAR_smb_anom(basin_idx,:);
smb.ens_smb_anom = smb.ens_smb_anom(basin_idx,:);

% Smooth the SMB anomaly timeseries
smb.ens_anom_smooth = movmean(smb.ens_smb_anom,60,2,'omitnan');
smb.HIRHAM_anom_smooth = movmean(smb.HIRHAM_smb_anom,60,2,'omitnan');
smb.RACMO_anom_smooth = movmean(smb.RACMO_smb_anom,60,2,'omitnan');
smb.MAR_anom_smooth = movmean(smb.MAR_smb_anom,60,2,'omitnan');

% Set the anomaly error (assume 20%)
smb.ens_anom_err = abs(smb.ens_anom_smooth).*0.2; 
smb.HIRHAM_anom_err = abs(smb.HIRHAM_anom_smooth).*0.2; 
smb.RACMO_anom_err = abs(smb.RACMO_anom_smooth).*0.2; 
smb.MAR_anom_err = abs(smb.MAR_anom_smooth).*0.2; 

% Get full SMB
smb.ens = Qbal.Qbal_ens + smb.ens_anom_smooth;
smb.HIRHAM = Qbal.Qbal_HIRHAM + smb.HIRHAM_anom_smooth;
smb.RACMO = Qbal.Qbal_RACMO + smb.RACMO_anom_smooth;
smb.MAR = Qbal.Qbal_MAR + smb.MAR_anom_smooth;

% Get the full SMB error
smb.ens_err = zeros(size(smb.ens_anom_err)).*NaN;
smb.HIRHAM_err = zeros(size(smb.ens_anom_err)).*NaN;
smb.RACMO_err = zeros(size(smb.ens_anom_err)).*NaN;
smb.MAR_err = zeros(size(smb.ens_anom_err)).*NaN;
for i = 1:numel(basins.name)
    smb.ens_err(i,:) = rssq([repmat(Qbal.Qbal_err_ens(i),[1,length(time)]) ; smb.ens_anom_err(i,:)],1);
    smb.HIRHAM_err(i,:) = rssq([repmat(Qbal.Qbal_err_HIRHAM(i),[1,length(time)]) ; smb.HIRHAM_anom_err(i,:)],1);
    smb.RACMO_err(i,:) = rssq([repmat(Qbal.Qbal_err_RACMO(i),[1,length(time)]) ; smb.RACMO_anom_err(i,:)],1);
    smb.MAR_err(i,:) = rssq([repmat(Qbal.Qbal_err_MAR(i),[1,length(time)]) ; smb.MAR_anom_err(i,:)],1);
end
smb = rmfield(smb,{'HIRHAM_smb_anom','RACMO_smb_anom','MAR_smb_anom','ens_smb_anom',...
    'smb_time','HIRHAM_time','RACMO_time','MAR_time','ens_anom_smooth','HIRHAM_anom_smooth',...
    'RACMO_anom_smooth','MAR_anom_smooth','ens_anom_err','MAR_anom_err','HIRHAM_anom_err','RACMO_anom_err'});


%% discharge
% load
Q.FB = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_mean');
Q.FBA = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBedAdj.nc','discharge_mean');
Q.BM = ncread('../out/pub/discharge/nc/AIS_discharge_BedMachine.nc','discharge_mean');
Q.B2 = ncread('../out/pub/discharge/nc/AIS_discharge_BedMap2.nc','discharge_mean');
Q.FBse = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','discharge_standard_error');
Q.FBAse = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBedAdj.nc','discharge_standard_error');
Q.BMse = ncread('../out/pub/discharge/nc/AIS_discharge_BedMachine.nc','discharge_standard_error');
Q.B2se = ncread('../out/pub/discharge/nc/AIS_discharge_BedMap2.nc','discharge_standard_error');
Qtime = ncread('../out/pub/discharge/nc/AIS_discharge_FrankenBed.nc','time')+datenum(1950,1,1,0,0,0);

% add negligible random noise to time
Qtime = [time(1);Qtime];
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(size(Qtime));
Qtime = Qtime + r;
Qtime = sort(Qtime);
        

% interpolate discharge to SMB time
Q.FB_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.FBA_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.BM_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.B2_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.FBse_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.FBAse_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.BMse_interp = zeros([numel(basins.name),length(time)]).*NaN;
Q.B2se_interp = zeros([numel(basins.name),length(time)]).*NaN;
for i = 1:numel(basins.name)
    % FrankenBed
    ts = Q.FB(i,:);
    ts = [Qbal.Qbal_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.FB_interp(i,:) = ts;
    % FrankenBedAdj
    ts = Q.FBA(i,:);
    ts = [Qbal.Qbal_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.FBA_interp(i,:) = ts;
    % BedMachine
    ts = Q.BM(i,:);
    ts = [Qbal.Qbal_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.BM_interp(i,:) = ts;
    % Bedmap2
    ts = Q.B2(i,:);
    ts = [Qbal.Qbal_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.B2_interp(i,:) = ts;

    % uncertainty
    % FrankenBed
    ts = Q.FBse(i,:);
    ts = [Qbal.Qbal_err_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.FBse_interp(i,:) = ts;
    % FrankenBedAdj
    ts = Q.FBAse(i,:);
    ts = [Qbal.Qbal_err_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.FBAse_interp(i,:) = ts;
    % BedMachine
    ts = Q.BMse(i,:);
    ts = [Qbal.Qbal_err_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.BMse_interp(i,:) = ts;
    % Bedmap2
    ts = Q.B2se(i,:);
    ts = [Qbal.Qbal_err_ens(i),ts]; % assume balance discharge in 1979
    ts = interp1(Qtime,ts,time,'pchip');
    Q.B2se_interp(i,:) = ts;
end

%% calculate mass change. units: Gt/yr, every month
% - FrankenBed
MB.FB.ens = smb.ens-Q.FB_interp; 
MB.FB.HIRHAM = smb.HIRHAM-Q.FB_interp; 
MB.FB.RACMO = smb.RACMO-Q.FB_interp; 
MB.FB.MAR = smb.MAR-Q.FB_interp;
% - FrankenBedAdj
MB.FBA.ens = smb.ens-Q.FBA_interp; 
MB.FBA.HIRHAM = smb.HIRHAM-Q.FBA_interp; 
MB.FBA.RACMO = smb.RACMO-Q.FBA_interp; 
MB.FBA.MAR = smb.MAR-Q.FBA_interp;
% - BedMachine
MB.BM.ens = smb.ens-Q.BM_interp; 
MB.BM.HIRHAM = smb.HIRHAM-Q.BM_interp; 
MB.BM.RACMO = smb.RACMO-Q.BM_interp; 
MB.BM.MAR = smb.MAR-Q.BM_interp;
% - Bedmap2
MB.B2.ens = smb.ens-Q.B2_interp; 
MB.B2.HIRHAM = smb.HIRHAM-Q.B2_interp; 
MB.B2.RACMO = smb.RACMO-Q.B2_interp; 
MB.B2.MAR = smb.MAR-Q.B2_interp;

%% Calculate mass change error
% - FrankenBed
MB.FB.ens_err = rssq(cat(3,smb.ens_err,Q.FBse_interp),3); 
MB.FB.HIRHAM_err = rssq(cat(3,smb.HIRHAM_err,Q.FBse_interp),3); 
MB.FB.RACMO_err = rssq(cat(3,smb.RACMO_err,Q.FBse_interp),3); 
MB.FB.MAR_err = rssq(cat(3,smb.MAR_err,Q.FBse_interp),3); 
% - FrankenBedAdj
MB.FBA.ens_err = rssq(cat(3,smb.ens_err,Q.FBAse_interp),3); 
MB.FBA.HIRHAM_err = rssq(cat(3,smb.HIRHAM_err,Q.FBAse_interp),3); 
MB.FBA.RACMO_err = rssq(cat(3,smb.RACMO_err,Q.FBAse_interp),3); 
MB.FBA.MAR_err = rssq(cat(3,smb.MAR_err,Q.FBAse_interp),3); 
% - BedMachine
MB.BM.ens_err = rssq(cat(3,smb.ens_err,Q.BMse_interp),3); 
MB.BM.HIRHAM_err = rssq(cat(3,smb.HIRHAM_err,Q.BMse_interp),3); 
MB.BM.RACMO_err = rssq(cat(3,smb.RACMO_err,Q.BMse_interp),3); 
MB.BM.MAR_err = rssq(cat(3,smb.MAR_err,Q.BMse_interp),3); 
% - Bedmap2
MB.B2.ens_err = rssq(cat(3,smb.ens_err,Q.B2se_interp),3); 
MB.B2.HIRHAM_err = rssq(cat(3,smb.HIRHAM_err,Q.B2se_interp),3); 
MB.B2.RACMO_err = rssq(cat(3,smb.RACMO_err,Q.B2se_interp),3); 
MB.B2.MAR_err = rssq(cat(3,smb.MAR_err,Q.B2se_interp),3); 

%% Accumulate the mass change. units: Gt
% - FrankenBed
cMB.FB.ens = cumsum(MB.FB.ens./12,2); 
cMB.FB.HIRHAM = cumsum(MB.FB.HIRHAM./12,2); 
cMB.FB.RACMO = cumsum(MB.FB.RACMO./12,2); 
cMB.FB.MAR = cumsum(MB.FB.MAR./12,2); 
% - FrankenBedAdj
cMB.FBA.ens = cumsum(MB.FBA.ens./12,2); 
cMB.FBA.HIRHAM = cumsum(MB.FBA.HIRHAM./12,2); 
cMB.FBA.RACMO = cumsum(MB.FBA.RACMO./12,2); 
cMB.FBA.MAR = cumsum(MB.FBA.MAR./12,2); 
% - BedMachine
cMB.BM.ens = cumsum(MB.BM.ens./12,2); 
cMB.BM.HIRHAM = cumsum(MB.BM.HIRHAM./12,2); 
cMB.BM.RACMO = cumsum(MB.BM.RACMO./12,2); 
cMB.BM.MAR = cumsum(MB.BM.MAR./12,2); 
% - Bedmap2
cMB.B2.ens = cumsum(MB.B2.ens./12,2); 
cMB.B2.HIRHAM = cumsum(MB.B2.HIRHAM./12,2); 
cMB.B2.RACMO = cumsum(MB.B2.RACMO./12,2); 
cMB.B2.MAR = cumsum(MB.B2.MAR./12,2); 

%% Accumulate the errors - root sum square of the annual errors
dt = datetime(time,'convertFrom','datenum');
% - FrankenBed
cMB.FB.ens_err = calcCumulativeError(dt,MB.FB.ens_err);
cMB.FB.HIRHAM_err = calcCumulativeError(dt,MB.FB.HIRHAM_err);
cMB.FB.RACMO_err = calcCumulativeError(dt,MB.FB.RACMO_err);
cMB.FB.MAR_err = calcCumulativeError(dt,MB.FB.MAR_err);
% - FrankenBedAdj
cMB.FBA.ens_err = calcCumulativeError(dt,MB.FBA.ens_err);
cMB.FBA.HIRHAM_err = calcCumulativeError(dt,MB.FBA.HIRHAM_err);
cMB.FBA.RACMO_err = calcCumulativeError(dt,MB.FBA.RACMO_err);
cMB.FBA.MAR_err = calcCumulativeError(dt,MB.FBA.MAR_err);
% - BedMachine
cMB.BM.ens_err = calcCumulativeError(dt,MB.BM.ens_err);
cMB.BM.HIRHAM_err = calcCumulativeError(dt,MB.BM.HIRHAM_err);
cMB.BM.RACMO_err = calcCumulativeError(dt,MB.BM.RACMO_err);
cMB.BM.MAR_err = calcCumulativeError(dt,MB.BM.MAR_err);
% - Bedmap2
cMB.B2.ens_err = calcCumulativeError(dt,MB.B2.ens_err);
cMB.B2.HIRHAM_err = calcCumulativeError(dt,MB.B2.HIRHAM_err);
cMB.B2.RACMO_err = calcCumulativeError(dt,MB.B2.RACMO_err);
cMB.B2.MAR_err = calcCumulativeError(dt,MB.B2.MAR_err);

%% Save
MB.time = time;
save('../out/pub/mass_balance/MB.mat','MB','cMB','basins','Q','smb','IMBIE','time');


%% Plot

% colors
cmap = cbrewer('qual','Set1',4);
FB_col = 'k';
FBA_col = cmap(2,:);
BM_col = cmap(3,:);
B2_col = cmap(4,:);

% Cumulative mass change
figure('color','w','units','inches','Position',[1 0 5 5]);
subplot = @(m,n,p) subtightplot (m, n, p,[.01 0.01],[.05 .05],[.1 .1]);
idx = find(time>=IMBIE.time(1));
for i = 1:4
    p1 = subplot(4,1,i);
    %shadedErrorBar(time(idx),cMB.FB.ens(i,idx)-cMB.FB.ens(i,idx(1)),cMB.FB.ens_err(i,idx)-cMB.FB.ens_err(i,idx(1)),'lineProps',{'color',FB_col}); datetick('x'); hold on
    shadedErrorBar(time(idx),cMB.FBA.ens(i,idx)-cMB.FBA.ens(i,idx(1)),cMB.FBA.ens_err(i,idx)-cMB.FBA.ens_err(i,idx(1)),'lineProps',{'color',FBA_col}); datetick('x'); hold on
    %shadedErrorBar(time(idx),cMB.BM.ens(i,idx)-cMB.BM.ens(i,idx(1)),cMB.BM.ens_err(i,idx)-cMB.BM.ens_err(i,idx(1)),'lineProps',{'color',BM_col}); 
    %shadedErrorBar(time(idx),cMB.B2.ens(i,idx)-cMB.B2.ens(i,idx(1)),cMB.B2.ens_err(i,idx)-cMB.B2.ens_err(i,idx(1)),'lineProps',{'color',B2_col}); 
    plot(IMBIE.time,IMBIE.cMB(i,:),'-k');
    plot(IMBIE.time,IMBIE.cMB(i,:)+IMBIE.cMB_err(i,:),'--k');
    plot(IMBIE.time,IMBIE.cMB(i,:)-IMBIE.cMB_err(i,:),'--k');
    set(gca,'XLim',[IMBIE.time(1) time(end)]);
    box on
    if i < 4
        set(gca,'XTickLabel',{' '});
    end
end

% Mass balance
figure('color','w','units','inches','Position',[1 0 5 5]);
subplot = @(m,n,p) subtightplot (m, n, p,[.03 0.01],[.05 .05],[.1 .1]);
idx = find(time>=IMBIE.time(1));
for i = 1:4
    p1 = subplot(4,1,i);
    plot([IMBIE.time(1) time(end)],[0 0],'color',[.5 .5 .5]); datetick('x'); hold on
    plot(IMBIE.time,IMBIE.MB(i,:),'linestyle','-','color',rgb('orange'),'linewidth',2);
    plot(IMBIE.time,IMBIE.MB(i,:)+IMBIE.MB_err(i,:),'linestyle',':','color',rgb('orange'),'linewidth',2);
    plot(IMBIE.time,IMBIE.MB(i,:)-IMBIE.MB_err(i,:),'linestyle',':','color',rgb('orange'),'linewidth',2);
    shadedErrorBar(time(idx),MB.B2.ens(i,idx),MB.B2.ens_err(i,idx),'lineProps',{'color',B2_col,'linewidth',2}); 
    shadedErrorBar(time(idx),MB.BM.ens(i,idx),MB.BM.ens_err(i,idx),'lineProps',{'color',BM_col,'linewidth',2}); 
    shadedErrorBar(time(idx),MB.FB.ens(i,idx),MB.FB.ens_err(i,idx),'lineProps',{'color',FB_col,'linewidth',2}); %datetick('x'); hold on
    shadedErrorBar(time(idx),MB.FBA.ens(i,idx),MB.FBA.ens_err(i,idx),'lineProps',{'color',FBA_col,'linewidth',2}); %datetick('x'); hold on
    plot([IMBIE.time(1) time(end)],[0 0],'color',[.5 .5 .5]); datetick('x'); hold on
    plot(IMBIE.time,IMBIE.MB(i,:),'linestyle','-','color',rgb('orange'),'linewidth',1);
    plot(IMBIE.time,IMBIE.MB(i,:)+IMBIE.MB_err(i,:),'linestyle',':','color',rgb('orange'),'linewidth',1);
    plot(IMBIE.time,IMBIE.MB(i,:)-IMBIE.MB_err(i,:),'linestyle',':','color',rgb('orange'),'linewidth',1);
    set(gca,'XLim',[IMBIE.time(1) time(end)]);
    box on
    if i == 2 || i == 4; set(gca,'YAxisLocation','right'); end
    if i < 4
        set(gca,'XTickLabel',{' '});
    end
    if i == 1; text(datenum(1996,7,1),-300,'Antarctica','color','k','fontname','Helvetica','fontweight','bold'); end
    if i == 2; text(datenum(1996,7,1),-300,'West Antarctica','color','k','fontname','Helvetica','fontweight','bold'); end
    if i == 3; text(datenum(1996,7,1),-140,'East Antarctica','color','k','fontname','Helvetica','fontweight','bold'); end
    if i == 4; text(datenum(1996,7,1),-70,'Peninsula','color','k','fontname','Helvetica','fontweight','bold'); end
end
export_fig('../out/pub/figures/mass_balance/mass_balance.png','-dpng','-r1200');


idx = time>=datenum(1996,4,1);
Gtime = time(idx);
% plot mass change timeseries
scatter(Gtime,MB(1,idx),20,'ks'); datetick('x');

% GRACE gives mass change relative to Jan 2011, so set that as zero
idx = find(Gtime>datenum(2011,1,1),1,'first');
cMB = cMB - cMB(:,idx);

% plot cumulative mass change
plot(Gtime,cMB(1,:),'ko'); datetick('x');

