function append_ice_velocity(migration_distance)

% add paths to libraries
if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
    addpath(genpath('/resstore/b0133/Data/Velocity'));
end

%% Load flux gate

g = load('../out/pub/gates/flux_gates_migrated.mat');
migration_distances = 0:200:3000; % metres
migration_pixels = find(migration_distances == migration_distance); % metres
gate_x = g.gate_x_migrate{migration_pixels};
gate_y = g.gate_y_migrate{migration_pixels};
gate_nan = isnan(gate_x); % indices of gate nans
clear g


%% Load raw velocity

load(['../out/int/gate_data/gate_vel_raw_linear_migrate_' num2str(migration_distance) 'm_compiled.mat']);

%% Extract new monthly velocities

% Get list of velocity mosaics
sp_list = dir('/resstore/b0133/SID/Antarctica/IV/S1/output/mosaic/monthly_corrected/**/sp_corrected.tif');

% Get times
dates = zeros([numel(sp_list),2]).*NaN;
for i = 1:numel(sp_list)
    d = extractAfter(sp_list(i).folder,'monthly_corrected/');
    dates(i,:) = [datenum(d(1:8),'yyyymmdd') , datenum(d(10:17),'yyyymmdd')];
end
md = nanmean(dates,2);

% Get dates since we last updated the velocity timeseries
keep = md>time_vel(end);
if sum(keep)==0; quit; end % quit matlab if there are no new dates
dates = dates(keep,:);
md = md(keep);
sp_list = sp_list(keep);

% Get image coordinates
[~,x,y] = geoimread([sp_list(1).folder '/vx_corrected.tif']);

% Pre-allocate output
sp = zeros([length(gate_x),numel(sp_list)]).*NaN;
sp_err = zeros([length(gate_x),numel(sp_list)]).*NaN;
source = ones([numel(sp_list),1]).*6;

% Loop through new files and extract at gate
for i = 1:numel(sp_list)
    disp(['Processing month ' num2str(i) ' of ' num2str(numel(sp_list)) ]);
    % vx
    data = double(geoimread([sp_list(i).folder '/sp_corrected.tif']));
    sp(:,i) = interp2(x,y,data,gate_x,gate_y,'linear');
    % error
    data = double(geoimread(['/resstore/b0133/SID/Antarctica/IV/S1/output/mosaic/monthly/' datestr(dates(i,1),'yyyymmdd') '_' datestr(dates(i,2),'yyyymmdd') '/err_clean.tif']));
    sp_err(:,i) = interp2(x,y,data,gate_x,gate_y,'linear');
end
clear data X Y x y

% Save new velocities
%save(['gate_vel_linear_migrate_' num2str(migration_distance) 'm_append_SID_corrected.mat'],'vx','vy','vx_err','vy_err','md','dates','vx_list','source');

%% Append new velocities

% Concatenate
sp_raw = [sp_raw, sp];
spE_raw = [spE_raw, sp_err];
time_vel = [time_vel, md(:)'];
source_vel = [source_vel, source(:)'];

% Sort
[~,iv] = sort(time_vel,'ascend');
time_vel = time_vel(iv);
sp_raw = sp_raw(:,iv);
spE_raw = spE_raw(:,iv);
source_vel = source_vel(:,iv);

%% Save updated raw velocity timeseries

% overwrite original velocities so that the dataset grows each month
save(['../out/int/gate_data/gate_vel_raw_linear_migrate_' num2str(migration_distance) 'm_compiled.mat'],'sp_raw','spE_raw','source_vel','time_vel');

quit;
