function calc_discharge_timeseries(migration_distance,H_dataset)

disp(['Processing flux gate ' num2str(migration_distance) 'm with ' H_dataset ]);

if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
end

%% Set physical parameters

% Ice density
rhoi = 917; % kg/m^3

% iterations for monte-carlo error
numErrorIterations = 100;

%% Load flux gate data

% Gate locations
g = load('../out/pub/gates/flux_gates_migrated.mat');
migration_distances = 0:200:3000; % metres
migration_pixels = find(migration_distances == migration_distance); % metres
gate_x = g.gate_x_migrate{migration_pixels};
gate_y = g.gate_y_migrate{migration_pixels};
[gate_lat,gate_lon] = ps2ll(gate_x,gate_y);
gate_nan = isnan(gate_x); % indices of gate nans
clear g

% Reference data (various versions of bed topograpghy, one surface, gate angle)
% Load thickness dataset
if strcmp(H_dataset,'FrankenBed') || strcmp(H_dataset,'FrankenBedAdj')
    ref = load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm_fast.mat'],...
    'gate_theta','gate_H_raw','gate_H_raw_err');
elseif strcmp(H_dataset,'BedMachine')
    ref = load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm_fast.mat'],...
    'gate_theta','gate_H_BM','gate_H_BM_err');
elseif strcmp(H_dataset,'BedMap2')
    ref = load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm_fast.mat'],...
    'gate_theta','gate_H_B2','gate_H_B2_err');
else
    disp('Unrecognised thickness dataset, exiting');
    quit
end

% ice velocity timeseries
V = load(['../out/int/gate_data/gate_vel_filled_migrate_' num2str(migration_distance) 'm.mat'],...
    'vx_filled','vy_filled','vxE_filled','vyE_filled','time_vel','filled','source_vel','filled_with_ref','alignments');

% gate-to-grounding line thickness change due to SMB
load('../out/int/gate_data/smb_mass_correction.mat'); % smb_mass; 2140x16 double; one correction for each basin and gate combination.

% dHdt at gate
dH0 = load(['../out/int/gate_data/gate_quadratic_dH_ts_' num2str(migration_distance) 'm.mat'],'gate_quad','gate_lin','gate_icept','gate_quad_err','gate_lin_err','gate_icept_err','gate_start_time');

% firn air content
FAC_FDMraw = load(['../out/int/gate_data/FAC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from IMAU FDM
FAC_GSFCraw = load(['../out/int/gate_data/FAC_GSFC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from GSFC with MERRA-2
FAC_GEMBraw = load(['../out/int/gate_data/FAC_GEMB_migrate_' num2str(migration_distance) 'm.mat']); % FAC from GEMB

% Calculate discharge at each velocity epoch, so set time here
time = V.time_vel;


%% Calculate cross-gate velocity
% The aim is to provide an estimate of the total discharge across the
% grounding line, so always use the 'filled' velocity. Later, we'll
% calculate how much of the discharge has been observed vs how much is
% 'filled'

% extrude gate theta for ease of computation
gate_theta = repmat(ref.gate_theta,[1,length(time)]); % angle of flux gate; gate never changes, so keep constant through time and assume no error in this angle

% main velocity
V.ctv = abs(V.vx_filled.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.vxE_filled(V.vxE_filled>5e3 | isnan(V.vxE_filled)) = abs(V.vx_filled(V.vxE_filled>5e3 | isnan(V.vxE_filled))).*0.1;
V.vyE_filled(V.vyE_filled>5e3 | isnan(V.vyE_filled)) = abs(V.vy_filled(V.vyE_filled>5e3 | isnan(V.vyE_filled))).*0.1;
V.vx_min = V.vx_filled-V.vxE_filled;
V.vx_max = V.vx_filled+V.vxE_filled;
V.vy_min = V.vy_filled-V.vyE_filled;
V.vy_max = V.vy_filled+V.vyE_filled;
V.ctv_vxmin = abs(V.vx_min.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.ctv_vxmax = abs(V.vx_max.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.ctv_vymin = abs(V.vx_filled.*sin(gate_theta) - V.vy_min.*cos(gate_theta));
V.ctv_vymax = abs(V.vx_filled.*sin(gate_theta) - V.vy_max.*cos(gate_theta));
V.ctvE = sqrt( ...
      (V.ctv-V.ctv_vxmin).^2 ...
    + (V.ctv-V.ctv_vxmax).^2 ...
    + (V.ctv-V.ctv_vymin).^2 ...
    + (V.ctv-V.ctv_vymax).^2);
V.ctvE(V.ctvE>1e3 | V.ctvE>V.ctv.*0.5) = V.ctv(V.ctvE>1e3 | V.ctvE>V.ctv.*0.5).*0.1; % check for unrealistic errors
V = rmfield(V,{'vx_min','vx_max','vy_min','vy_max','ctv_vxmin','ctv_vxmax','ctv_vymin','ctv_vymax','vx_filled','vy_filled','vxE_filled','vyE_filled'});

% without shift
Vns = load(['../out/int/gate_data/gate_vel_filled_migrate_' num2str(migration_distance) 'm_ns.mat'],...
    'vx_filled','vy_filled','vxE_filled','vyE_filled','time_vel','filled','source_vel','filled_with_ref','alignments'); % load it here to minimise memory demands
Vns.ctv = abs(Vns.vx_filled.*sin(gate_theta) - Vns.vy_filled.*cos(gate_theta));
Vns.vxE_filled(Vns.vxE_filled>5e3 | isnan(Vns.vxE_filled)) = abs(Vns.vx_filled(Vns.vxE_filled>5e3 | isnan(Vns.vxE_filled))).*0.1;
Vns.vyE_filled(Vns.vyE_filled>5e3 | isnan(Vns.vyE_filled)) = abs(Vns.vy_filled(Vns.vyE_filled>5e3 | isnan(Vns.vyE_filled))).*0.1;
Vns.vx_min = Vns.vx_filled-Vns.vxE_filled;
Vns.vx_max = Vns.vx_filled+Vns.vxE_filled;
Vns.vy_min = Vns.vy_filled-Vns.vyE_filled;
Vns.vy_max = Vns.vy_filled+Vns.vyE_filled;
Vns.ctv_vxmin = abs(Vns.vx_min.*sin(gate_theta) - Vns.vy_filled.*cos(gate_theta));
Vns.ctv_vxmax = abs(Vns.vx_max.*sin(gate_theta) - Vns.vy_filled.*cos(gate_theta));
Vns.ctv_vymin = abs(Vns.vx_filled.*sin(gate_theta) - Vns.vy_min.*cos(gate_theta));
Vns.ctv_vymax = abs(Vns.vx_filled.*sin(gate_theta) - Vns.vy_max.*cos(gate_theta));
Vns.ctvE = sqrt( ...
      (Vns.ctv-Vns.ctv_vxmin).^2 ...
    + (Vns.ctv-Vns.ctv_vxmax).^2 ...
    + (Vns.ctv-Vns.ctv_vymin).^2 ...
    + (Vns.ctv-Vns.ctv_vymax).^2);
Vns.ctvE(Vns.ctvE>1e3 | Vns.ctvE>Vns.ctv.*0.5) = Vns.ctv(Vns.ctvE>1e3 | Vns.ctvE>Vns.ctv.*0.5).*0.1; % check for unrealistic errors
Vns = rmfield(Vns,{'vx_min','vx_max','vy_min','vy_max','ctv_vxmin','ctv_vxmax','ctv_vymin','ctv_vymax','vx_filled','vy_filled','vxE_filled','vyE_filled'});

clear gate_theta

%% Evaluate elevation change fit at velocity epochs

% convert velocity time to required format to evaluate fit
dH0.t = (time-datenum(1990,1,15))/365; % 1:1/12:32; % years from 1991
dH0.t_sampling_original = 140/365;
dH0.t_sampling_hires = 1/12;

% Calculate dH timeseries
dH = (dH0.gate_quad.*dH0.t.^2) + (dH0.gate_lin.*dH0.t) + dH0.gate_icept;
dH_upper = ((dH0.gate_quad+dH0.gate_quad_err).*dH0.t.^2) + ((dH0.gate_lin+dH0.gate_lin_err).*dH0.t) + (dH0.gate_icept+dH0.gate_icept_err);
dH_lower = ((dH0.gate_quad-dH0.gate_quad_err).*dH0.t.^2) + ((dH0.gate_lin-dH0.gate_lin_err).*dH0.t) + (dH0.gate_icept-dH0.gate_icept_err);
dH_err = (dH0.t_sampling_hires  / dH0.t_sampling_original) .* ((abs(dH_upper - dH) + abs(dH - dH_lower)) ./ 2);
clear dH_upper dH_lower

% Set reference time to May 9th 2015 (time of reference thickness)
[~,idx] = findNearest(time, datenum(2015,5,9));
dH = dH - dH(:,idx);

% Keep static (reference) thickness south of 81.5
dH(dH0.gate_start_time>=2010) = 0;
dH_err(dH0.gate_start_time>=2010) = 0;

% Some places the gate dH error is unrealistically large
dH_err(dH_err>abs(dH).*0.5) = abs(dH(dH_err>abs(dH).*0.5)).*0.1;
dH_err(dH_err<abs(dH).*0.05) = abs(dH(dH_err<abs(dH).*0.05)).*0.1;
dH_err(isnan(dH_err)) = abs(dH(isnan(dH_err))).*0.1;

% Set to NaN at gaps in flux gate
dH(gate_nan,:) = NaN;
dH_err(gate_nan,:) = NaN;

clear dH0 idx

%% Interpolate firn corrections to velocity epochs
% pre-allocate
FAC_FDM = zeros(size(V.ctv)).*NaN;
FAC_GSFC = zeros(size(V.ctv)).*NaN;
FAC_GEMB = zeros(size(V.ctv)).*NaN;
% loop through gate pixels
for j = 1:length(gate_x)
    if isfinite(gate_x(j))
        
        % load FAC and interpolate to velocity/discharge timeseries
        FAC_ts = FAC_FDMraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_FDMraw.fac_time,FAC_ts,time);
        % As the velocity time-series extends beyond the FAC timeseries,
        % the more recent FAC time points will be NaN. Fill those with the
        % monthly FAC climatology.
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_FDMraw.fac_monthly_clim(j,M);
        end
        FAC_FDM(j,:) = FAC_ts;

        % Repeat for GSFC firn model
        FAC_ts = FAC_GSFCraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_GSFCraw.fac_time,FAC_ts,time);
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_GSFCraw.fac_monthly_clim(j,M);
        end
        FAC_GSFC(j,:) = FAC_ts;

        % Repeat for GEMB firn model
        FAC_ts = FAC_GEMBraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_GEMBraw.fac_time,FAC_ts,time);
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_GEMBraw.fac_monthly_clim(j,M);
        end
        FAC_GEMB(j,:) = FAC_ts;
    end
end
FDM_clim = FAC_FDMraw.fac_clim;
clear FAC_ts FAC_FDMraw FAC_GSFCraw FAC_GEMBraw M isn j

%% Calculate time varying ice thickness
    
% Load thickness dataset
if strcmp(H_dataset,'FrankenBed')
    H = double(repmat(ref.gate_H_raw(:),[1,length(time)]));
    He = double(repmat(ref.gate_H_raw_err(:),[1,length(time)]));
elseif strcmp(H_dataset,'FrankenBedAdj')
    load(['../out/int/derived/thickness_adjustment_' num2str(migration_distance) 'm.mat'],'H_change');
    H = double(repmat(ref.gate_H_raw(:)+H_change,[1,length(time)])); % note, we're modifying the 'raw' FrankenBed thickness here based on the differnece between the FrankenBed discharge and the discharge required to match observed dHdt from 1996-2021
    He = double(repmat(ref.gate_H_raw_err(:),[1,length(time)]));
elseif strcmp(H_dataset,'BedMachine')
    H  = double(repmat(ref.gate_H_BM(:), [1,length(time)]));
    He  = double(repmat(ref.gate_H_BM_err(:), [1,length(time)]));
elseif strcmp(H_dataset,'BedMap2')
    H  = double(repmat(ref.gate_H_B2(:), [1,length(time)]));
    He  = double(repmat(ref.gate_H_B2_err(:), [1,length(time)]));
else
    disp('Unrecognised thickness dataset, exiting');
    quit
end
    
% Check for missing thickness error
He(isnan(He) & isfinite(H)) = H(isnan(He) & isfinite(H)).*0.3; % assume very large (30%) error in these places (should be none, but do it just in case)

% Apply thickness change
dH = double(dH);
H_static = H;      % 'static'; no corrections applied
H_NDH    = H;      % 'No dH'; apply everything except time-varying dH
H_NF     = H + dH; % 'no firn'; apply everything except a firn correction
H        = H + dH; % apply everything
clear dH

% Add dH error to relevant thickness datasets, and initialise others.
% Also assume 10% error in the firn air content correction
dH_err      = double(dH_err);
FAC_FDM     = double(FAC_FDM);
FAC_GSFC    = double(FAC_GSFC);
FAC_GEMB    = double(FAC_GEMB);
He_static   = He;
He_NDH      = sqrt(He.^2 + (FAC_FDM.*0.1).^2);
He_NF       = sqrt(He.^2 + dH_err.^2);
He_GSFC     = sqrt(He.^2 + dH_err.^2 + (FAC_GSFC.*0.1).^2); % initialise error for discharge using GSFC firn model; include dH error
He_GEMB     = sqrt(He.^2 + dH_err.^2 + (FAC_GEMB.*0.1).^2); % initialise error for discharge using GEMB firn model; include dH error
He          = sqrt(He.^2 + dH_err.^2 + (FAC_FDM .*0.1).^2);
clear dH_err

% Apply firn correction
%   - the baseline thickness estimates above already have the
%   climatological firn content removed, so we need to add that back on
%   then apply the time-varying firn corrections
FDM_clim = double(FDM_clim);
H        = H        + repmat(FDM_clim,[1,length(time)]) - FAC_FDM;
H_NDH    = H_NDH    + repmat(FDM_clim,[1,length(time)]) - FAC_FDM;
H_GSFC   = H        + repmat(FDM_clim,[1,length(time)]) - FAC_GSFC;
H_GEMB   = H        + repmat(FDM_clim,[1,length(time)]) - FAC_GEMB;
H_NF     = H_NF     + repmat(FDM_clim,[1,length(time)]); % add the climatological firn air content back in for this one
H_static = H_static + repmat(FDM_clim,[1,length(time)]); % and this one
clear FDM_clim FAC_FDM FAC_GSFC FAC_GEMB
    
% After all those corrections, check that we haven't created any
% unrealistic ice thickness
H(H<0)               = 0;
H_static(H_static<0) = 0;
H_NDH(H_NDH<0)       = 0;
H_NF(H_NF<0)         = 0;
H_GSFC(H_GSFC<0)     = 0;
H_GEMB(H_GEMB<0)     = 0;

% Finally, make sure that we have NaNs in the right place
H(gate_nan,:)        = NaN;
H_static(gate_nan,:) = NaN;
H_NDH(gate_nan,:)    = NaN;
H_NF(gate_nan,:)     = NaN;
H_GSFC(gate_nan,:)   = NaN;
H_GEMB(gate_nan,:)   = NaN;

%% Calculate discharge

% Get flux gate sections
sections = regionprops(~isnan(gate_x), gate_x,'pixelIdxList','PixelValues');
dx = zeros(size(gate_x)).*NaN;
for j = 1:numel(sections)
    % Get section data
    sect_lat = gate_lat(sections(j).PixelIdxList);
    sect_lon = gate_lon(sections(j).PixelIdxList);
    d = pathdistps(sect_lat(:),sect_lon(:)); % Distance along flux gate
    dx(sections(j).PixelIdxList) = gradient(d); % Distance between each flux gate data point
end
clear sect_lat sect_lon d
dx = repmat(dx(:),[1,length(time)]);

% Calculate discharge
%   - regular velocity
Q              = V.ctv.*H.*dx.*rhoi.*1e-12;        %   - merged bed, REMA surface, without adjustments
Q_static       = V.ctv.*H_static.*dx.*rhoi.*1e-12; %   - no thickness corrections applied at all
Q_NDH          = V.ctv.*H_NDH.*dx.*rhoi.*1e-12;      %   - no dH applied 
Q_NF           = V.ctv.*H_NF.*dx.*rhoi.*1e-12;       %   - no firn correction applied
Q_GSFC         = V.ctv.*H_GSFC.*dx.*rhoi.*1e-12;     %   - GSFC firn correction applied
Q_GEMB         = V.ctv.*H_GEMB.*dx.*rhoi.*1e-12;     %   - GEMB firn correction applied
%   - no shift
Qns            = Vns.ctv.*H.*dx.*rhoi.*1e-12;        %   - merged bed, REMA surface, without adjustments

% Calculate the erros in each pixel as the standard deviation of
% discharge +/- the error in a monte-carlo style approach
disp('starting monte carlo');
tic
Q_vel_err = zeros(size(Q)).*NaN;
Qns_vel_err = zeros(size(Q)).*NaN;
Q_H_err = zeros(size(Q)).*NaN;
Q_static_H_err = zeros(size(Q)).*NaN;
Q_NDH_H_err = zeros(size(Q)).*NaN;
Q_NF_H_err = zeros(size(Q)).*NaN;
Q_GSFC_H_err = zeros(size(Q)).*NaN;
Q_GEMB_H_err = zeros(size(Q)).*NaN;
for j = 1:length(time)
    perc = j/length(time).*100;
    disp(['Monte Carlo progress: ' num2str(perc) '%'])
    velDischarge         = zeros([length(gate_x),1]).*NaN;
    velDischarge_ns      = zeros([length(gate_x),1]).*NaN;
    thickDischarge       = zeros([length(gate_x),1]).*NaN;
    thickDischargeStatic = zeros([length(gate_x),1]).*NaN;
    thickDischargeNDH    = zeros([length(gate_x),1]).*NaN;
    thickDischargeNF     = zeros([length(gate_x),1]).*NaN;
    thickDischargeGSFC   = zeros([length(gate_x),1]).*NaN;
    thickDischargeGEMB   = zeros([length(gate_x),1]).*NaN;
    for k = 1:numErrorIterations
        v = unifrnd(-V.ctvE(:,j),V.ctvE(:,j),[length(gate_x),1]);
        velDischarge(:,k) = (V.ctv(:,j)+v).*H(:,j).*dx(:,j).*rhoi.*1e-12;
        v = unifrnd(-Vns.ctvE(:,j),Vns.ctvE(:,j),[length(gate_x),1]);
        velDischarge_ns(:,k) = (Vns.ctv(:,j)+v).*H(:,j).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He(:,j),He(:,j),[length(gate_x),1]);
        thickDischarge(:,k) = V.ctv(:,j).*(H(:,j)+h).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He_static(:,j),He_static(:,j),[length(gate_x),1]);
        thickDischargeStatic(:,k) = V.ctv(:,j).*(H_static(:,j)+h).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He_NDH(:,j),He_NDH(:,j),[length(gate_x),1]);
        thickDischargeNDH(:,k) = V.ctv(:,j).*(H_NDH(:,j)+h).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He_NF(:,j),He_NF(:,j),[length(gate_x),1]);
        thickDischargeNF(:,k) = V.ctv(:,j).*(H_NF(:,j)+h).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He_GSFC(:,j),He_GSFC(:,j),[length(gate_x),1]);
        thickDischargeGSFC(:,k) = V.ctv(:,j).*(H_GSFC(:,j)+h).*dx(:,j).*rhoi.*1e-12;
        h = unifrnd(-He_GEMB(:,j),He_GEMB(:,j),[length(gate_x),1]);
        thickDischargeGEMB(:,k) = V.ctv(:,j).*(H_GEMB(:,j)+h).*dx(:,j).*rhoi.*1e-12;
    end
    Q_vel_err(:,j)      = nanstd(velDischarge,[],2);
    Qns_vel_err(:,j)    = nanstd(velDischarge_ns,[],2);
    Q_H_err(:,j)        = nanstd(thickDischarge,[],2);
    Q_static_H_err(:,j) = nanstd(thickDischargeStatic,[],2);
    Q_NDH_H_err(:,j)    = nanstd(thickDischargeNDH,[],2);
    Q_NF_H_err(:,j)     = nanstd(thickDischargeNF,[],2);
    Q_GSFC_H_err(:,j)   = nanstd(thickDischargeGSFC,[],2);
    Q_GEMB_H_err(:,j)   = nanstd(thickDischargeGEMB,[],2);
end
clear v j h k velDischarge velDischarge_ns thickDischarge thickDischarge* He He_static He_NDH He_NF He_GSFC He_GEMB dx
disp('done monte carlo in');
toc
disp(' ');

% Errors (assume velocity and thickness errors are independent)
disp('Calculating total errors')
tic
Q_err        = sqrt(Q_vel_err.^2 + Q_H_err.^2);        %   - all corrections
Qns_err      = sqrt(Qns_vel_err.^2 + Q_H_err.^2);      %   - all corrections with no shift velocity
Q_static_err = sqrt(Q_vel_err.^2 + Q_static_H_err.^2); %   - static (no thickness corrections applied)
Q_NDH_err    = sqrt(Q_vel_err.^2 + Q_NDH_H_err.^2);    %   - NDH (no dH correction)
Q_NF_err     = sqrt(Q_vel_err.^2 + Q_NF_H_err.^2);     %   - NF (no firn correction)
Q_GSFC_err   = sqrt(Q_vel_err.^2 + Q_GSFC_H_err.^2);   %   - GSFC firn correction, rather than FDM
Q_GEMB_err   = sqrt(Q_vel_err.^2 + Q_GEMB_H_err.^2);   %   - GEMB firn correction, rather than FDM
clear Qns_vel_err Q_static_H_err Q_NDH_H_err Q_NF_H_err Q_GSFC_H_err Q_GEMB_H_err
disp('done calculating errors')
toc
disp(' ');
    
% Make single precision
disp('Making single precision');
tic
Q = single(Q);
Qns = single(Qns);
Q_err = single(Q_err);
Qns_err = single(Qns_err);
Q_vel_err = single(Q_vel_err);
Q_H_err = single(Q_H_err);
Q_static = single(Q_static);
Q_static_err = single(Q_static_err);
Q_NF = single(Q_NF);
Q_NF_err = single(Q_NF_err);
Q_NDH = single(Q_NDH);
Q_NDH_err = single(Q_NDH_err);
Q_GSFC = single(Q_GSFC);
Q_GSFC_err = single(Q_GSFC_err);
Q_GEMB = single(Q_GEMB);
Q_GEMB_err = single(Q_GEMB_err);
disp('done single precision')
toc
disp(' ');

%% Save pixel-based discharge time-series

disp('Saving pixel-based discharge');
tic
filled = V.filled;
velocity_source = V.source_vel;
save(['../out/int/discharge/pixel/discharge_timeseries_migrate_' num2str(migration_distance) 'm_' H_dataset '.mat'],...
    'Q','Q_err','Q_vel_err','Q_H_err','Qns','Qns_err',...
    'Q_static','Q_static_err',...
    'Q_NF','Q_NF_err',...
    'Q_NDH','Q_NDH_err',...
    'Q_GSFC','Q_GSFC_err',...
    'Q_GEMB','Q_GEMB_err',...
    'time','filled','velocity_source'); 
	
toc
disp(' ');

%% Calculate timeseries for each basin

% basin info
load('../out/int/basins/all_basins.mat');

% Pre-allocate
disp('pre-allocating basin-scale discharge variables');
tic
%   - discharge
bQ            = zeros([numel(basins.name),length(time)]).*NaN;
bQ_ns         = zeros([numel(basins.name),length(time)]).*NaN;
bQ_static     = zeros([numel(basins.name),length(time)]).*NaN;
bQ_NF         = zeros([numel(basins.name),length(time)]).*NaN;
bQ_NDH        = zeros([numel(basins.name),length(time)]).*NaN;
bQ_GSFC       = zeros([numel(basins.name),length(time)]).*NaN;
bQ_GEMB       = zeros([numel(basins.name),length(time)]).*NaN;
bQE           = zeros([numel(basins.name),length(time)]).*NaN;
bQE_ns        = zeros([numel(basins.name),length(time)]).*NaN;
bQE_vel       = zeros([numel(basins.name),length(time)]).*NaN;
bQE_H         = zeros([numel(basins.name),length(time)]).*NaN;
bQE_static    = zeros([numel(basins.name),length(time)]).*NaN;
bQE_NF        = zeros([numel(basins.name),length(time)]).*NaN;
bQE_NDH       = zeros([numel(basins.name),length(time)]).*NaN;
bQE_GSFC      = zeros([numel(basins.name),length(time)]).*NaN;
bQE_GEMB      = zeros([numel(basins.name),length(time)]).*NaN;
%   - proportion of discharge observed
bQp        = zeros([numel(basins.name),length(time)]).*NaN;
bQp_static = zeros([numel(basins.name),length(time)]).*NaN;
bQp_NF     = zeros([numel(basins.name),length(time)]).*NaN;
bQp_NDH    = zeros([numel(basins.name),length(time)]).*NaN;
bQp_GSFC   = zeros([numel(basins.name),length(time)]).*NaN;
bQp_GEMB   = zeros([numel(basins.name),length(time)]).*NaN;
toc
disp(' ');

% Make copies where filled discharge is set to nan
disp('making copied for filled discharge stats');
tic
filled = logical(filled);
Qf = Q;
Qf(filled) = 0;
Qf_static = Q_static;
Qf_static(filled) = 0;
Qf_NF = Q_NF;
Qf_NF(filled) = 0;
Qf_NDH = Q_NDH;
Qf_NDH(filled) = 0;
Qf_GSFC = Q_GSFC;
Qf_GSFC(filled) = 0;
Qf_GEMB = Q_GEMB;
Qf_GEMB(filled) = 0;
toc
disp(' ');

% There is a nan splitting the main ice sheet from wilkins
% island. To root-sum-square the errors for basins including that gap, 
% we need to set that nan to 0...
disp('setting gate nan to zero');
tic
isn = isnan(Q_err);
Q_err(isn) = 0;
Qns_err(isn) = 0;
Q_vel_err(isn) = 0;
Q_H_err(isn) = 0;
Q_static_err(isn) = 0;
Q_NF_err(isn) = 0;
Q_NDH_err(isn) = 0;
Q_GSFC_err(isn) = 0;
Q_GEMB_err(isn) = 0;
toc
disp(' ');

% calculate basin-integrated discharge
disp('integrating discharge for each basin');
tic
for j = 1:numel(basins.name)
    disp(['progress ' num2str(j) '/' num2str(numel(basins.name))]);
    load(['../out/int/gate_data/pixels_in_basins/' num2str(migration_distance) 'm/' num2str(j) '_' basins.name{j} '_tf_pixels_in_basin_' num2str(migration_distance) 'm.mat'],'tf');
    % Some of the peninsula basins don't intersect any gate. Some
    % upstream migrated gates end up coming out of the basin, we can
    % have empty tf sometimes. Keep those as nan?
    if ~isempty(tf)
        % Discharge
        %   - main
        bQ(j,:)      = nansum(Q(tf,:),1);
        Qmax         = nansum(Q(tf,:)+Q_err(tf,:));
        Qmin         = nansum(Q(tf,:)-Q_err(tf,:));
        bQE(j,:)     = abs(abs(Qmax-bQ(j,:)) + abs(bQ(j,:)-Qmin))./2;
        Qmax         = nansum(Q(tf,:)+Q_vel_err(tf,:));
        Qmin         = nansum(Q(tf,:)-Q_vel_err(tf,:));
        bQE_vel(j,:) = abs(abs(Qmax-bQ(j,:)) + abs(bQ(j,:)-Qmin))./2;
        Qmax         = nansum(Q(tf,:)+Q_H_err(tf,:));
        Qmin         = nansum(Q(tf,:)-Q_H_err(tf,:));
        bQE_H(j,:) = abs(abs(Qmax-bQ(j,:)) + abs(bQ(j,:)-Qmin))./2;
        %   - main no shift
        bQ_ns(j,:)   = nansum(Qns(tf,:),1);
        Qmax         = nansum(Qns(tf,:)+Qns_err(tf,:));
        Qmin         = nansum(Qns(tf,:)-Qns_err(tf,:));
        bQE_ns(j,:)  = abs(abs(Qmax-bQ_ns(j,:)) + abs(bQ_ns(j,:)-Qmin))./2;
        %   - corrections
        % -- static
        bQ_static(j,:)  = nansum(Q_static(tf,:),1);
        Qmax            = nansum(Q_static(tf,:)+Q_static_err(tf,:));
        Qmin            = nansum(Q_static(tf,:)-Q_static_err(tf,:));
        bQE_static(j,:) = abs(abs(Qmax-bQ_static(j,:)) + abs(bQ_static(j,:)-Qmin))./2;
        % -- no firn
        bQ_NF(j,:)  = nansum(Q_NF(tf,:),1);
        Qmax        = nansum(Q_NF(tf,:)+Q_NF_err(tf,:));
        Qmin        = nansum(Q_NF(tf,:)-Q_NF_err(tf,:));
        bQE_NF(j,:) = abs(abs(Qmax-bQ_NF(j,:)) + abs(bQ_NF(j,:)-Qmin))./2;
        % -- no dH
        bQ_NDH(j,:)  = nansum(Q_NDH(tf,:),1);
        Qmax         = nansum(Q_NDH(tf,:)+Q_NDH_err(tf,:));
        Qmin         = nansum(Q_NDH(tf,:)-Q_NDH_err(tf,:));
        bQE_NDH(j,:) = abs(abs(Qmax-bQ_NDH(j,:)) + abs(bQ_NDH(j,:)-Qmin))./2;
        %   - alternative firn models
        % -- GSFC
        bQ_GSFC(j,:)  = nansum(Q_GSFC(tf,:),1);
        Qmax          = nansum(Q_GSFC(tf,:)+Q_GSFC_err(tf,:));
        Qmin          = nansum(Q_GSFC(tf,:)-Q_GSFC_err(tf,:));
        bQE_GSFC(j,:) = abs(abs(Qmax-bQ_GSFC(j,:)) + abs(bQ_GSFC(j,:)-Qmin))./2;
        % -- GEMB
        bQ_GEMB(j,:)  = nansum(Q_GEMB(tf,:),1);
        Qmax          = nansum(Q_GEMB(tf,:)+Q_GEMB_err(tf,:));
        Qmin          = nansum(Q_GEMB(tf,:)-Q_GEMB_err(tf,:));
        bQE_GEMB(j,:) = abs(abs(Qmax-bQ_GEMB(j,:)) + abs(bQ_GEMB(j,:)-Qmin))./2;

        % Proportion of discharge observed
        bQp(j,:)        = nansum(Qf(tf,:),1)       ./bQ(j,:)       .*100;
        bQp_static(j,:) = nansum(Qf_static(tf,:),1)./bQ_static(j,:).*100;
        bQp_NF(j,:)     = nansum(Qf_NF(tf,:),1)    ./bQ_NF(j,:)    .*100;
        bQp_NDH(j,:)    = nansum(Qf_NDH(tf,:),1)   ./bQ_NDH(j,:)   .*100;
        %   - alternative firn models
        bQp_GSFC(j,:) = nansum(Qf_GSFC(tf,:),1)./bQ_GSFC(j,:).*100;
        bQp_GEMB(j,:) = nansum(Qf_GEMB(tf,:),1)./bQ_GEMB(j,:).*100;
    end

    % Re-instate the NaN
    Q_err(isn) = NaN;
    Qns_err(isn) = NaN;
    Q_static_err(isn) = NaN;
    Q_NF_err(isn) = NaN;
    Q_NDH_err(isn) = NaN;
    Q_GSFC_err(isn) = NaN;
    Q_GEMB_err(isn) = NaN;

    % Add the SMB-induced mass changes to the relevant discharge
    % time-series (everything except 'static')
    bQ(j,:)      = bQ(j,:)      + smb_mass(j,migration_pixels);
    bQ_ns(j,:)   = bQ_ns(j,:)   + smb_mass(j,migration_pixels);
    bQ_NF(j,:)   = bQ_NF(j,:)   + smb_mass(j,migration_pixels);
    bQ_NDH(j,:)  = bQ_NDH(j,:)  + smb_mass(j,migration_pixels);
    bQ_GSFC(j,:) = bQ_GSFC(j,:) + smb_mass(j,migration_pixels);
    bQ_GEMB(j,:) = bQ_GEMB(j,:) + smb_mass(j,migration_pixels);
    
end
disp('done integrating discharge')
toc
disp(' ');

% prep time for outlier removal and smoothing
a = -0.000001;
b = 0.000001;
r = a + (b-a).*rand(size(time));
tmp = time + r;
tmp = sort(tmp);

disp(' ');
disp(' ')
disp(' ');
disp('removing outliers from discharge timeseries');
tic
% Remove outliers
c=-1/(sqrt(2)*erfcinv(3/2)); % scaling factor for outlier removal 
bQ_detrend = detrend(bQ',1,'omitnan','samplepoints',datetime(tmp,'convertfrom','datenum'))'; % remove linear trend from all timeseries; takes a while...
sMAD = c.*nanmedian(abs(bQ_detrend-nanmedian(bQ_detrend,2)),2);
removed = zeros(size(bQ));
removed(abs(bQ_detrend-nanmedian(bQ_detrend,2)) > sMAD.*3) = 1;
bQ(logical(removed))        = NaN;
bQ_ns(logical(removed))     = NaN;
bQ_static(logical(removed)) = NaN;
bQ_NF(logical(removed))     = NaN;
bQ_NDH(logical(removed))    = NaN;
bQ_GSFC(logical(removed))   = NaN;
bQ_GEMB(logical(removed))   = NaN;
tf = isoutlier(bQ,'movmedian',30*4,2,'samplepoints',tmp,'ThresholdFactor', 2); % outliers are those more than 3-scaled MADs from the median in a 90-day moving window
bQ(tf)        = NaN;
bQ_ns(tf)     = NaN;
bQ_static(tf) = NaN;
bQ_NF(tf)     = NaN;
bQ_NDH(tf)    = NaN;
bQ_GSFC(tf)   = NaN;
bQ_GEMB(tf)   = NaN;
toc
disp(' ');

disp('filling gaps')
tic
% Fill new gaps
for j = 1:size(bQ,1)
    bQ(j,:)        = interp1gap(tmp, bQ(j,:),       tmp,'spline','extrap',NaN);
    bQ_ns(j,:)     = interp1gap(tmp, bQ_ns(j,:),    tmp,'spline','extrap',NaN);
    bQ_static(j,:) = interp1gap(tmp, bQ_static(j,:),tmp,'spline','extrap',NaN);
    bQ_NF(j,:)     = interp1gap(tmp, bQ_NF(j,:),    tmp,'spline','extrap',NaN);
    bQ_NDH(j,:)    = interp1gap(tmp, bQ_NDH(j,:),   tmp,'spline','extrap',NaN);
    bQ_GSFC(j,:)   = interp1gap(tmp, bQ_GSFC(j,:),  tmp,'spline','extrap',NaN);
    bQ_GEMB(j,:)   = interp1gap(tmp, bQ_GEMB(j,:),  tmp,'spline','extrap',NaN);
    bQ(j,:)         = fillmissing(fillmissing(bQ(j,:),        'next'), 'previous');
    bQ_ns(j,:)      = fillmissing(fillmissing(bQ_ns(j,:),     'next'), 'previous');
    bQ_static(j,:)  = fillmissing(fillmissing(bQ_static(j,:), 'next'), 'previous');
    bQ_NF(j,:)      = fillmissing(fillmissing(bQ_NF(j,:),     'next'), 'previous');
    bQ_NDH(j,:)     = fillmissing(fillmissing(bQ_NDH(j,:),    'next'), 'previous');
    bQ_GSFC(j,:)    = fillmissing(fillmissing(bQ_GSFC(j,:),   'next'), 'previous');
    bQ_GEMB(j,:)    = fillmissing(fillmissing(bQ_GEMB(j,:),   'next'), 'previous');
end
toc
disp(' ');

disp('smoothing timeseries');
tic
% Smooth the basin-scale time-series a bit
%   - smooth with a moving mean filter just larger than one year, but
%   only apply prior to July 2015
target = time<=datenum(2015,7,1);
bQ(:,target)        = movmean(bQ(:,target),       30*13,2,'omitnan','samplepoints',tmp(target));
bQ_ns(:,target)     = movmean(bQ_ns(:,target),    30*13,2,'omitnan','samplepoints',tmp(target));
bQ_static(:,target) = movmean(bQ_static(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
bQ_NF(:,target)     = movmean(bQ_NF(:,target),    30*13,2,'omitnan','samplepoints',tmp(target));
bQ_NDH(:,target)    = movmean(bQ_NDH(:,target),   30*13,2,'omitnan','samplepoints',tmp(target));
bQ_GSFC(:,target)   = movmean(bQ_GSFC(:,target),  30*13,2,'omitnan','samplepoints',tmp(target));
bQ_GEMB(:,target)   = movmean(bQ_GEMB(:,target),  30*13,2,'omitnan','samplepoints',tmp(target));
%   - smooth with a moving mean filter just larger than three months
bQ        = movmean(bQ,       94,2,'omitnan','samplepoints',tmp);
bQ_static = movmean(bQ_static,94,2,'omitnan','samplepoints',tmp);
bQ_NF     = movmean(bQ_NF,    94,2,'omitnan','samplepoints',tmp);
bQ_NDH    = movmean(bQ_NDH,   94,2,'omitnan','samplepoints',tmp);
bQ_GSFC   = movmean(bQ_GSFC,  94,2,'omitnan','samplepoints',tmp);
bQ_GEMB   = movmean(bQ_GEMB,  94,2,'omitnan','samplepoints',tmp);
toc
disp(' ');

% make single precision
disp('making single precision')
tic
bQ = single(bQ); bQE = single(bQE); bQp = single(bQp); bQE_vel = single(bQE_vel); bQE_H = single(bQE_H); bQ_ns = single(bQ_ns); bQE_ns = single(bQE_ns);
bQ_static = single(bQ_static); bQE_static = single(bQE_static); bQp_static = single(bQp_static); 
bQ_NF = single(bQ_NF); bQE_NF = single(bQE_NF); bQp_NF = single(bQp_NF); 
bQ_NDH = single(bQ_NDH); bQE_NDH = single(bQE_NDH); bQp_NDH = single(bQp_NDH); 
bQ_GSFC = single(bQ_GSFC); bQE_GSFC = single(bQE_GSFC); bQp_GSFC = single(bQp_GSFC); 
bQ_GEMB = single(bQ_GEMB); bQE_GEMB = single(bQE_GEMB); bQp_GEMB = single(bQp_GEMB); 
toc
disp(' ');

% Save as .mat file
disp('saving as .mat file');
tic
save(['../out/int/discharge/basin/basin_discharge_migrate_' num2str(migration_distance) 'm_' H_dataset '_all_basins.mat'],...
    'bQ','bQE','bQp','bQE_vel','bQE_H','bQ_ns','bQE_ns',...
    'bQ_static','bQE_static','bQp_static',...
    'bQ_NF','bQE_NF','bQp_NF',...
    'bQ_NDH','bQE_NDH','bQp_NDH',...
    'bQ_GSFC','bQE_GSFC','bQp_GSFC',...
    'bQ_GEMB','bQE_GEMB','bQp_GEMB',...
    'time','velocity_source');
toc
disp(' ');



%% Adjust bed elevation to match observed mass change

% Calculate elevation adjustment using FrankenBed discharge
if strcmp(H_dataset,'FrankenBed')
    
    disp('Calculating FrankenBedAdj elevation adjustment')
    tic
    
    % Pre-allocate output (set them to zero, so if this fails we just
    % won't modify the thickness at all)
    H_change = zeros(size(gate_x));
    H_new = zeros(size(gate_x));
    basin_H_change = zeros([numel(basins.name),1]);
    basin_H_inverted = zeros([numel(basins.name),1]);
    basin_H_orig = zeros([numel(basins.name),1]);
    
    % load inverted discharge
    inv = load('../out/int/discharge/inverted/inverted_discharge.mat'); % this is the discharge we need to match the timeseries of dH/dt

    % Use average observed discharge during overlap with SMB and dHdt
    keep = time>=datenum(1996,1,1) & time<=datenum(2021,12,31);
    
    % Get the average velocity, thickness and discharge during the
    % period of overlap
    H = double(repmat(ref.gate_H_raw(:),[1,length(time)]));
    H(H<0) = 1e-3; % set very small thickness in any places with impossible thickness
    V_avg = nanmean(V.ctv(:,keep),2);
    H_avg = nanmean(H(:,keep),2);
    H_avg(H_avg<0) = 1e-3; % somehow, some negative thickness still persists occasionally, so do this again
    
    %  Calculate distance between flux gate points
    sections = regionprops(~isnan(gate_x), gate_x,'pixelIdxList','PixelValues');
    dx = zeros(size(gate_x)).*NaN;
    for j = 1:numel(sections)
        sect_lat = gate_lat(sections(j).PixelIdxList);
        sect_lon = gate_lon(sections(j).PixelIdxList);
        d = pathdistps(sect_lat(:),sect_lon(:)); % Distance along flux gate
        dx(sections(j).PixelIdxList) = gradient(d); % Distance between each flux gate data point
    end
    
    % Keep a copy of the original thickness
    H_avg0 = H_avg;
    
    % Calculate thickness scaling for MEaSUREs regional basins
    basin_idx = find(basins.type_ID==1);
    for j = 1:numel(basin_idx)
        
        % Only bother if the discharge inversion worked
        if inv.Qinv(basin_idx(j))>0 && isfinite(inv.Qinv(basin_idx(j))) && ~isinf(inv.Qinv(basin_idx(j)))

            % Get location of pixels in this basin
            tf = inpoly2([gate_x(:),gate_y(:)],[basins.x{basin_idx(j)},basins.y{basin_idx(j)}]);
            sections = regionprops(tf==1, tf,'pixelIdxList','PixelValues');
            if numel(sections)>1
                tf = double(tf);
                tf(tf==0) = NaN;
                tf = fillmissing(double(tf),'linear','EndValues','none','MaxGap',500); % limit the infill size to try and stop it wrapping round...
                tf(isnan(tf)) = 0;
                tf = logical(tf);
            end
            
            if ~isempty(tf)

                % Calculate the discharge in this basin (time average
                % during the above time period)
                pixel_Q = V_avg(tf).*H_avg(tf).*dx(tf).*rhoi.*1e-12;
                basin_Q  = nansum(pixel_Q);
                resid_Q = basin_Q - inv.Qinv(basin_idx(j));
                percResid_Q = resid_Q / inv.Qinv(basin_idx(j)) * 100;
                percResid_Q0 = percResid_Q; % keep a copy of the original

                % Iteratively adjust the thickness in these pixels until we
                % hit the right discharge (or within 0.1% of it)
                numIterations = 0;
                while abs(percResid_Q)>0.1 && numIterations<2000 
                    % Initially assume that we have too little discharge
                    % and we need to get some more by increasing ice
                    % thickness
                    fac = 1;
                    if resid_Q > 0 % but if we have too much discharge, set a factor to make things the right sign
                        fac = -1;
                    end
                    % Prevent numerical errors
                    if percResid_Q == 0 || isinf(percResid_Q)
                        percResid_Q = 1e-5;
                    end
                    % Modify the thickness
                    H_avg(tf) = H_avg(tf) + (H_avg(tf).*(abs(percResid_Q)/100*fac));
                    % Recalculate discharge
                    pixel_Q = V_avg(tf).*H_avg(tf).*dx(tf).*rhoi.*1e-12;
                    basin_Q  = nansum(pixel_Q);
                    resid_Q = basin_Q - inv.Qinv(basin_idx(j));
                    percResid_Q = resid_Q / inv.Qinv(basin_idx(j)) * 100;
                    % Prevent numerical errors
                    if percResid_Q == 0 || isinf(percResid_Q)
                        percResid_Q = 1e-5;
                    end
                    % increment counter
                    numIterations = numIterations+1;
                end
                if numIterations==2000
                    disp(' ');
                    disp(['Warning maximum number of iterations reached for basin ' basins.name{basin_idx(j)}]);
                    disp(['Final percentage residual was: ' num2str(percResid_Q) ' % (compared to the initial: ' num2str(percResid_Q0) '%)']);
                else
                    disp(['Final percentage residual for ' basins.name{basin_idx(j)} ' was: ' num2str(percResid_Q) ' % (compared to the initial: ' num2str(percResid_Q0) '%). NumIters = ' num2str(numIterations) ]);
                end

                % Keep track of new thicknesses
                H_new(tf) = H_avg(tf);
                H_change(tf) = H_avg(tf) - H_avg0(tf); % positive means we have increased ice thickness to achive the necessary discharge
                basin_H_change(basin_idx(j)) = nanmean(H_change(tf));  
                basin_H_inverted(basin_idx(j)) = nanmean(H_avg(tf));
                basin_H_orig(basin_idx(j)) = nanmean(H_avg0(tf));
                
            end
            
        end
        
    end
    
    % Clear some space
    clear inv keep V_avg H_avg H_avg0 basin_idx j inv tf sections pixel_Q basin_Q resid_Q percResid_Q numIterations

    
    % Make sure we haven't created any weird thickness or produce
    % unrealistically large changes in thickness
    remove = isinf(H_change) | isnan(H_change) | H_new<=0;
    H_change(remove) = 0; 
        
    % Keep a record of the thickness multiplier
    save(['../out/int/derived/thickness_adjustment_' num2str(migration_distance) 'm.mat'],'H_change','H_new','basin_H_change','basin_H_inverted','basin_H_orig');

    toc
    disp(' ')
    disp(' ')
    disp(' ');
end

disp(' ')
disp(' ')
disp(' ');


% END OF MAIN FUNCTION
end


