% merge_discharge_timeseries_SID_corrected

clear
close all
clc

if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
    addpath(genpath('../'));
end

%% Preparation

% set netcdf compression factor
compression_factor = 5; % 0-9

% Set gate migration distance
migration_distance = 0:200:3000; % metres

% basins and gates
load('../out/int/basins/all_basins.mat');

% Balance discharge
balance_discharge = load('../out/int/discharge/balance/balance_discharge.mat');

% Thickness datasets
H_datasets = {'FrankenBed','FrankenBedAdj','BedMachine','BedMap2'};

%% Write all_basins output 
% Some of the basins on the Peninsula islands aren't covered by the gates,
% thickness or velocity datasets used here, so have no discharge, but were
% included in the initial list of islands. I will exclude those ones from
% the final, published output, but for documentation purposes it can be
% useful to write them here first

keep = logical(ones([numel(basins.name),1]));
for i = 1:numel(H_datasets)
    discharge_to_netcdf(H_datasets{i},basins,migration_distance,['AIS_discharge_' H_datasets{i} '_all_basins.nc'],compression_factor,keep);
end

%% Write main output

% Find basins with discharge
[basins,balance_discharge,keep] = trim_basins;

% Balance discharge
balance_discharge_to_netcdf(basins,balance_discharge,'balance_discharge.nc');
balance_discharge_to_csv(basins,balance_discharge,'balance_discharge.csv');

% Basins
basins_to_netcdf(basins,'basins.nc',compression_factor);
basins_to_shapefile(basins,'basins.shp');

% Discharge - all variables
for i = 1:numel(H_datasets)
    discharge_to_netcdf_allVar(H_datasets{i},basins,migration_distance,['AIS_discharge_' H_datasets{i} '_allVar.nc'],compression_factor,keep);
end

% Discharge
for i = 1:numel(H_datasets)
    discharge_to_netcdf(H_datasets{i},basins,migration_distance,['AIS_discharge_' H_datasets{i} '.nc'],compression_factor,keep);
	discharge_to_csv(H_datasets{i},basins,migration_distance,['AIS_discharge_' H_datasets{i} '.csv'],keep);
end

% Save FrankenBedAdj info
gates = load('../out/pub/gates/flux_gates_migrated');
FrankenBedAdj_to_netcdf(gates,migration_distance,'FrankenBedAdj.nc',compression_factor);

quit;


